'''
   MFEM example 3

   See c++ version in the MFEM library for more detail 
'''
from mfem import path
import mfem.ser as mfem
from mfem.ser import intArray
from os.path import expanduser, join
import numpy as np
from numpy import sin, array

freq = 1.0
kappa = np.pi*freq

static_cond = False
order = 1
meshfile = expanduser(join(path, 'data', 'beam-tet.mesh'))

mesh = mfem.Mesh(meshfile, 1,1)

dim = mesh.Dimension()
sdim= mesh.SpaceDimension()

class E_exact(mfem.VectorPyCoefficient):
    def __init__(self):
       
       mfem.VectorPyCoefficient.__init__(self, dim)
    def EvalValue(self, x):
       return (sin(kappa * x[1]),
               sin(kappa * x[2]),
               sin(kappa * x[0]))
class f_exact(mfem.VectorPyCoefficient):
    def __init__(self):
       mfem.VectorPyCoefficient.__init__(self, dim)
    def EvalValue(self, x):   
       return ((1 + kappa**2)*sin(kappa * x[1]),
               (1 + kappa**2)*sin(kappa * x[2]),
               (1 + kappa**2)*sin(kappa * x[0]))

#   3. Refine the mesh to increase the resolution. In this example we do
#      'ref_levels' of uniform refinement. We choose 'ref_levels' to be the
#      largest number that gives a final mesh with no more than 50,000
#      elements.

ref_levels = int(np.floor(np.log(50000./mesh.GetNE())/np.log(2.)/dim))
for x in range(ref_levels):
   mesh.UniformRefinement();
mesh.ReorientTetMesh();

#  4. Define a finite element space on the mesh. Here we use the Nedelec
#     finite elements of the specified order.

fec = mfem.ND_FECollection(order, dim)
fespace = mfem.FiniteElementSpace(mesh, fec)

print("Number of finite element unknowns: " + str(fespace.GetTrueVSize()))

# 5. Determine the list of true (i.e. conforming) essential boundary dofs.
#    In this example, the boundary conditions are defined by marking all
#    the boundary attributes from the mesh as essential (Dirichlet) and
#    converting them to a list of true dofs.

ess_tdof_list = intArray();
if mesh.bdr_attributes.Size():
    ess_bdr = intArray(mesh.bdr_attributes.Max())
    ess_bdr = intArray([1]*mesh.bdr_attributes.Max())
    fespace.GetEssentialTrueDofs(ess_bdr, ess_tdof_list);

# 6. Set up the linear form b(.) which corresponds to the right-hand side
#    of the FEM linear system, which in this case is (f,phi_i) where f is
#    given by the function f_exact and phi_i are the basis functions in the
#    finite element fespace.

b = mfem.LinearForm(fespace);
f = f_exact()
dd = mfem.VectorFEDomainLFIntegrator(f);
b.AddDomainIntegrator(dd)
b.Assemble();

# 7. Define the solution vector x as a finite element grid function
#    corresponding to fespace. Initialize x by projecting the exact
#    solution. Note that only values from the boundary edges will be used
#    when eliminating the non-homogeneous boundary condition to modify the
#    r.h.s. vector b.

#from mfem.examples.ex3 import E_exact_cb
x = mfem.GridFunction(fespace)
E = E_exact()
x.ProjectCoefficient(E);

# 8. Set up the bilinear form corresponding to the EM diffusion operator
#       curl muinv curl + sigma I, by adding the curl-curl and the mass domain
#       integrators.

muinv = mfem.ConstantCoefficient(1.0);
sigma = mfem.ConstantCoefficient(1.0);
a = mfem.BilinearForm(fespace);
a.AddDomainIntegrator(mfem.CurlCurlIntegrator(muinv));
a.AddDomainIntegrator(mfem.VectorFEMassIntegrator(sigma));

# 9. Assemble the bilinear form and the corresponding linear system,
#       applying any necessary transformations such as: eliminating boundary
#       conditions, applying conforming constraints for non-conforming AMR,
#       static condensation, etc.


if (static_cond):  a.EnableStaticCondensation()
a.Assemble();

A = mfem.SparseMatrix()
B = mfem.Vector()
X = mfem.Vector()
a.FormLinearSystem(ess_tdof_list, x, b, A, X, B);
## Here, original version calls hegith, which is not
## defined in the header...!?
print("Size of linear system: " + str(A.Size())) 
import sys
sys.stdout.flush()
# 10. Solve 
M = mfem.GSSmoother(A)
mfem.PCG(A, M, B, X, 1, 500, 1e-12, 0.0);
sys.stdout.flush()

# 11. Recover the solution as a finite element grid function.
a.RecoverFEMSolution(X, b, x);

# 12. Compute and print the L^2 norm of the error.
import sys
sys.stdout.write("|| E_h - E ||_{L^2} = " + str(x.ComputeL2Error(E))+"\n")

mesh.Print('refined.mesh', 8)
x.Save('sol.gf', 8)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _pnonlinearform
else:
    import _pnonlinearform

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _pnonlinearform.SWIG_PyInstanceMethod_New
_swig_new_static_method = _pnonlinearform.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

MFEM_VERSION = _pnonlinearform.MFEM_VERSION
MFEM_VERSION_STRING = _pnonlinearform.MFEM_VERSION_STRING
MFEM_VERSION_TYPE = _pnonlinearform.MFEM_VERSION_TYPE
MFEM_VERSION_TYPE_RELEASE = _pnonlinearform.MFEM_VERSION_TYPE_RELEASE
MFEM_VERSION_TYPE_DEVELOPMENT = _pnonlinearform.MFEM_VERSION_TYPE_DEVELOPMENT
MFEM_VERSION_MAJOR = _pnonlinearform.MFEM_VERSION_MAJOR
MFEM_VERSION_MINOR = _pnonlinearform.MFEM_VERSION_MINOR
MFEM_VERSION_PATCH = _pnonlinearform.MFEM_VERSION_PATCH
MFEM_HYPRE_VERSION = _pnonlinearform.MFEM_HYPRE_VERSION
import mfem._par.vector
import mfem._par.array
import mfem._par.mem_manager
import mfem._par.nonlinearform
import mfem._par.operators
import mfem._par.fespace
import mfem._par.coefficient
import mfem._par.globals
import mfem._par.matrix
import mfem._par.intrules
import mfem._par.sparsemat
import mfem._par.densemat
import mfem._par.eltrans
import mfem._par.fe
import mfem._par.geom
import mfem._par.mesh
import mfem._par.sort_pairs
import mfem._par.ncmesh
import mfem._par.vtk
import mfem._par.element
import mfem._par.table
import mfem._par.hash
import mfem._par.vertex
import mfem._par.gridfunc
import mfem._par.bilininteg
import mfem._par.fe_coll
import mfem._par.lininteg
import mfem._par.linearform
import mfem._par.handle
import mfem._par.hypre
import mfem._par.restriction
import mfem._par.bilinearform
import mfem._par.nonlininteg
import mfem._par.blockoperator
import mfem._par.pfespace
import mfem._par.pmesh
import mfem._par.pncmesh
import mfem._par.communication
import mfem._par.sets
import mfem._par.pgridfunc
class ParNonlinearForm(mfem._par.nonlinearform.NonlinearForm):
    r"""Proxy of C++ mfem::ParNonlinearForm class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, pf):
        r"""__init__(ParNonlinearForm self, ParFiniteElementSpace pf) -> ParNonlinearForm"""
        _pnonlinearform.ParNonlinearForm_swiginit(self, _pnonlinearform.new_ParNonlinearForm(pf))

    def ParFESpace(self):
        r"""ParFESpace(ParNonlinearForm self) -> ParFiniteElementSpace"""
        return _pnonlinearform.ParNonlinearForm_ParFESpace(self)
    ParFESpace = _swig_new_instance_method(_pnonlinearform.ParNonlinearForm_ParFESpace)

    def GetParGridFunctionEnergy(self, x):
        r"""GetParGridFunctionEnergy(ParNonlinearForm self, Vector x) -> double"""
        return _pnonlinearform.ParNonlinearForm_GetParGridFunctionEnergy(self, x)
    GetParGridFunctionEnergy = _swig_new_instance_method(_pnonlinearform.ParNonlinearForm_GetParGridFunctionEnergy)

    def GetEnergy(self, *args):
        r"""
        GetEnergy(ParNonlinearForm self, ParGridFunction x) -> double
        GetEnergy(ParNonlinearForm self, Vector x) -> double
        """
        return _pnonlinearform.ParNonlinearForm_GetEnergy(self, *args)
    GetEnergy = _swig_new_instance_method(_pnonlinearform.ParNonlinearForm_GetEnergy)

    def Mult(self, x, y):
        r"""Mult(ParNonlinearForm self, Vector x, Vector y)"""
        return _pnonlinearform.ParNonlinearForm_Mult(self, x, y)
    Mult = _swig_new_instance_method(_pnonlinearform.ParNonlinearForm_Mult)

    def GetLocalGradient(self, x):
        r"""GetLocalGradient(ParNonlinearForm self, Vector x) -> SparseMatrix"""
        return _pnonlinearform.ParNonlinearForm_GetLocalGradient(self, x)
    GetLocalGradient = _swig_new_instance_method(_pnonlinearform.ParNonlinearForm_GetLocalGradient)

    def GetGradient(self, x):
        r"""GetGradient(ParNonlinearForm self, Vector x) -> Operator"""
        return _pnonlinearform.ParNonlinearForm_GetGradient(self, x)
    GetGradient = _swig_new_instance_method(_pnonlinearform.ParNonlinearForm_GetGradient)

    def SetGradientType(self, tid):
        r"""SetGradientType(ParNonlinearForm self, mfem::Operator::Type tid)"""
        return _pnonlinearform.ParNonlinearForm_SetGradientType(self, tid)
    SetGradientType = _swig_new_instance_method(_pnonlinearform.ParNonlinearForm_SetGradientType)

    def Update(self):
        r"""Update(ParNonlinearForm self)"""
        return _pnonlinearform.ParNonlinearForm_Update(self)
    Update = _swig_new_instance_method(_pnonlinearform.ParNonlinearForm_Update)
    __swig_destroy__ = _pnonlinearform.delete_ParNonlinearForm

# Register ParNonlinearForm in _pnonlinearform:
_pnonlinearform.ParNonlinearForm_swigregister(ParNonlinearForm)

class ParBlockNonlinearForm(mfem._par.nonlinearform.BlockNonlinearForm):
    r"""Proxy of C++ mfem::ParBlockNonlinearForm class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def GetEnergy(self, x):
        r"""GetEnergy(ParBlockNonlinearForm self, Vector x) -> double"""
        return _pnonlinearform.ParBlockNonlinearForm_GetEnergy(self, x)
    GetEnergy = _swig_new_instance_method(_pnonlinearform.ParBlockNonlinearForm_GetEnergy)

    def __init__(self, *args):
        r"""
        __init__(ParBlockNonlinearForm self) -> ParBlockNonlinearForm
        __init__(ParBlockNonlinearForm self, mfem::Array< mfem::ParFiniteElementSpace * > & pf) -> ParBlockNonlinearForm
        """
        _pnonlinearform.ParBlockNonlinearForm_swiginit(self, _pnonlinearform.new_ParBlockNonlinearForm(*args))

    def ParFESpace(self, *args):
        r"""
        ParFESpace(ParBlockNonlinearForm self, int k) -> ParFiniteElementSpace
        ParFESpace(ParBlockNonlinearForm self, int k) -> ParFiniteElementSpace
        """
        return _pnonlinearform.ParBlockNonlinearForm_ParFESpace(self, *args)
    ParFESpace = _swig_new_instance_method(_pnonlinearform.ParBlockNonlinearForm_ParFESpace)

    def SetParSpaces(self, pf):
        r"""SetParSpaces(ParBlockNonlinearForm self, mfem::Array< mfem::ParFiniteElementSpace * > & pf)"""
        return _pnonlinearform.ParBlockNonlinearForm_SetParSpaces(self, pf)
    SetParSpaces = _swig_new_instance_method(_pnonlinearform.ParBlockNonlinearForm_SetParSpaces)

    def SetEssentialBC(self, bdr_attr_is_ess, rhs):
        r"""SetEssentialBC(ParBlockNonlinearForm self, mfem::Array< mfem::Array< int > * > const & bdr_attr_is_ess, mfem::Array< mfem::Vector * > & rhs)"""
        return _pnonlinearform.ParBlockNonlinearForm_SetEssentialBC(self, bdr_attr_is_ess, rhs)
    SetEssentialBC = _swig_new_instance_method(_pnonlinearform.ParBlockNonlinearForm_SetEssentialBC)

    def Mult(self, x, y):
        r"""Mult(ParBlockNonlinearForm self, Vector x, Vector y)"""
        return _pnonlinearform.ParBlockNonlinearForm_Mult(self, x, y)
    Mult = _swig_new_instance_method(_pnonlinearform.ParBlockNonlinearForm_Mult)

    def GetLocalGradient(self, x):
        r"""GetLocalGradient(ParBlockNonlinearForm self, Vector x) -> BlockOperator"""
        return _pnonlinearform.ParBlockNonlinearForm_GetLocalGradient(self, x)
    GetLocalGradient = _swig_new_instance_method(_pnonlinearform.ParBlockNonlinearForm_GetLocalGradient)

    def GetGradient(self, x):
        r"""GetGradient(ParBlockNonlinearForm self, Vector x) -> BlockOperator"""
        return _pnonlinearform.ParBlockNonlinearForm_GetGradient(self, x)
    GetGradient = _swig_new_instance_method(_pnonlinearform.ParBlockNonlinearForm_GetGradient)

    def SetGradientType(self, tid):
        r"""SetGradientType(ParBlockNonlinearForm self, mfem::Operator::Type tid)"""
        return _pnonlinearform.ParBlockNonlinearForm_SetGradientType(self, tid)
    SetGradientType = _swig_new_instance_method(_pnonlinearform.ParBlockNonlinearForm_SetGradientType)
    __swig_destroy__ = _pnonlinearform.delete_ParBlockNonlinearForm

# Register ParBlockNonlinearForm in _pnonlinearform:
_pnonlinearform.ParBlockNonlinearForm_swigregister(ParBlockNonlinearForm)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _geom
else:
    import _geom

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _geom.SWIG_PyInstanceMethod_New
_swig_new_static_method = _geom.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._par.intrules
import mfem._par.array
import mfem._par.mem_manager
import mfem._par.densemat
import mfem._par.vector
import mfem._par.operators
import mfem._par.matrix
class Geometry(object):
    r"""Proxy of C++ mfem::Geometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    INVALID = _geom.Geometry_INVALID
    
    POINT = _geom.Geometry_POINT
    
    SEGMENT = _geom.Geometry_SEGMENT
    
    TRIANGLE = _geom.Geometry_TRIANGLE
    
    SQUARE = _geom.Geometry_SQUARE
    
    TETRAHEDRON = _geom.Geometry_TETRAHEDRON
    
    CUBE = _geom.Geometry_CUBE
    
    PRISM = _geom.Geometry_PRISM
    
    NUM_GEOMETRIES = _geom.Geometry_NUM_GEOMETRIES
    
    NumGeom = _geom.Geometry_NumGeom
    
    MaxDim = _geom.Geometry_MaxDim
    
    Name = property(_geom.Geometry_Name_get, _geom.Geometry_Name_set, doc=r"""Name : a(mfem::Geometry::NumGeom).p.q(const).char""")

    def __init__(self):
        r"""__init__(Geometry self) -> Geometry"""
        _geom.Geometry_swiginit(self, _geom.new_Geometry())
    __swig_destroy__ = _geom.delete_Geometry

    def GetVertices(self, GeomType):
        r"""GetVertices(Geometry self, int GeomType) -> IntegrationRule"""
        return _geom.Geometry_GetVertices(self, GeomType)
    GetVertices = _swig_new_instance_method(_geom.Geometry_GetVertices)

    def GetCenter(self, GeomType):
        r"""GetCenter(Geometry self, int GeomType) -> IntegrationPoint"""
        return _geom.Geometry_GetCenter(self, GeomType)
    GetCenter = _swig_new_instance_method(_geom.Geometry_GetCenter)

    @staticmethod
    def GetRandomPoint(GeomType, ip):
        r"""GetRandomPoint(int GeomType, IntegrationPoint ip)"""
        return _geom.Geometry_GetRandomPoint(GeomType, ip)
    GetRandomPoint = _swig_new_static_method(_geom.Geometry_GetRandomPoint)

    @staticmethod
    def CheckPoint(*args):
        r"""
        CheckPoint(int GeomType, IntegrationPoint ip) -> bool
        CheckPoint(int GeomType, IntegrationPoint ip, double eps) -> bool
        """
        return _geom.Geometry_CheckPoint(*args)
    CheckPoint = _swig_new_static_method(_geom.Geometry_CheckPoint)

    @staticmethod
    def ProjectPoint(*args):
        r"""
        ProjectPoint(int GeomType, IntegrationPoint beg, IntegrationPoint end) -> bool
        ProjectPoint(int GeomType, IntegrationPoint ip) -> bool
        """
        return _geom.Geometry_ProjectPoint(*args)
    ProjectPoint = _swig_new_static_method(_geom.Geometry_ProjectPoint)

    def GetGeomToPerfGeomJac(self, GeomType):
        r"""GetGeomToPerfGeomJac(Geometry self, int GeomType) -> DenseMatrix"""
        return _geom.Geometry_GetGeomToPerfGeomJac(self, GeomType)
    GetGeomToPerfGeomJac = _swig_new_instance_method(_geom.Geometry_GetGeomToPerfGeomJac)

    def GetPerfGeomToGeomJac(self, GeomType):
        r"""GetPerfGeomToGeomJac(Geometry self, int GeomType) -> DenseMatrix"""
        return _geom.Geometry_GetPerfGeomToGeomJac(self, GeomType)
    GetPerfGeomToGeomJac = _swig_new_instance_method(_geom.Geometry_GetPerfGeomToGeomJac)

    def GetPerfPointMat(self, GeomType, pm):
        r"""GetPerfPointMat(Geometry self, int GeomType, DenseMatrix pm)"""
        return _geom.Geometry_GetPerfPointMat(self, GeomType, pm)
    GetPerfPointMat = _swig_new_instance_method(_geom.Geometry_GetPerfPointMat)

    def JacToPerfJac(self, GeomType, J, PJ):
        r"""JacToPerfJac(Geometry self, int GeomType, DenseMatrix J, DenseMatrix PJ)"""
        return _geom.Geometry_JacToPerfJac(self, GeomType, J, PJ)
    JacToPerfJac = _swig_new_instance_method(_geom.Geometry_JacToPerfJac)

    def NumBdr(self, GeomType):
        r"""NumBdr(Geometry self, int GeomType) -> int"""
        return _geom.Geometry_NumBdr(self, GeomType)
    NumBdr = _swig_new_instance_method(_geom.Geometry_NumBdr)

# Register Geometry in _geom:
_geom.Geometry_swigregister(Geometry)
cvar = _geom.cvar
Geometry.NumBdrArray = _geom.cvar.Geometry_NumBdrArray
Geometry.Volume = _geom.cvar.Geometry_Volume
Geometry.Dimension = _geom.cvar.Geometry_Dimension
Geometry.DimStart = _geom.cvar.Geometry_DimStart
Geometry.NumVerts = _geom.cvar.Geometry_NumVerts
Geometry.NumEdges = _geom.cvar.Geometry_NumEdges
Geometry.NumFaces = _geom.cvar.Geometry_NumFaces

def Geometry_GetRandomPoint(GeomType, ip):
    r"""Geometry_GetRandomPoint(int GeomType, IntegrationPoint ip)"""
    return _geom.Geometry_GetRandomPoint(GeomType, ip)
Geometry_GetRandomPoint = _geom.Geometry_GetRandomPoint

def Geometry_CheckPoint(*args):
    r"""
    Geometry_CheckPoint(int GeomType, IntegrationPoint ip) -> bool
    Geometry_CheckPoint(int GeomType, IntegrationPoint ip, double eps) -> bool
    """
    return _geom.Geometry_CheckPoint(*args)
Geometry_CheckPoint = _geom.Geometry_CheckPoint

def Geometry_ProjectPoint(*args):
    r"""
    Geometry_ProjectPoint(int GeomType, IntegrationPoint beg, IntegrationPoint end) -> bool
    Geometry_ProjectPoint(int GeomType, IntegrationPoint ip) -> bool
    """
    return _geom.Geometry_ProjectPoint(*args)
Geometry_ProjectPoint = _geom.Geometry_ProjectPoint

class RefinedGeometry(object):
    r"""Proxy of C++ mfem::RefinedGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    Times = property(_geom.RefinedGeometry_Times_get, _geom.RefinedGeometry_Times_set, doc=r"""Times : int""")
    ETimes = property(_geom.RefinedGeometry_ETimes_get, _geom.RefinedGeometry_ETimes_set, doc=r"""ETimes : int""")
    RefPts = property(_geom.RefinedGeometry_RefPts_get, doc=r"""RefPts : mfem::IntegrationRule""")
    RefGeoms = property(_geom.RefinedGeometry_RefGeoms_get, _geom.RefinedGeometry_RefGeoms_set, doc=r"""RefGeoms : mfem::Array<(int)>""")
    RefEdges = property(_geom.RefinedGeometry_RefEdges_get, _geom.RefinedGeometry_RefEdges_set, doc=r"""RefEdges : mfem::Array<(int)>""")
    NumBdrEdges = property(_geom.RefinedGeometry_NumBdrEdges_get, _geom.RefinedGeometry_NumBdrEdges_set, doc=r"""NumBdrEdges : int""")
    Type = property(_geom.RefinedGeometry_Type_get, _geom.RefinedGeometry_Type_set, doc=r"""Type : int""")

    def __init__(self, NPts, NRefG, NRefE, NBdrE=0):
        r"""__init__(RefinedGeometry self, int NPts, int NRefG, int NRefE, int NBdrE=0) -> RefinedGeometry"""
        _geom.RefinedGeometry_swiginit(self, _geom.new_RefinedGeometry(NPts, NRefG, NRefE, NBdrE))
    __swig_destroy__ = _geom.delete_RefinedGeometry

# Register RefinedGeometry in _geom:
_geom.RefinedGeometry_swigregister(RefinedGeometry)

class GeometryRefiner(object):
    r"""Proxy of C++ mfem::GeometryRefiner class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(GeometryRefiner self) -> GeometryRefiner"""
        _geom.GeometryRefiner_swiginit(self, _geom.new_GeometryRefiner())

    def SetType(self, t):
        r"""SetType(GeometryRefiner self, int const t)"""
        return _geom.GeometryRefiner_SetType(self, t)
    SetType = _swig_new_instance_method(_geom.GeometryRefiner_SetType)

    def GetType(self):
        r"""GetType(GeometryRefiner self) -> int"""
        return _geom.GeometryRefiner_GetType(self)
    GetType = _swig_new_instance_method(_geom.GeometryRefiner_GetType)

    def Refine(self, Geom, Times, ETimes=1):
        r"""Refine(GeometryRefiner self, mfem::Geometry::Type Geom, int Times, int ETimes=1) -> RefinedGeometry"""
        return _geom.GeometryRefiner_Refine(self, Geom, Times, ETimes)
    Refine = _swig_new_instance_method(_geom.GeometryRefiner_Refine)

    def RefineInterior(self, Geom, Times):
        r"""RefineInterior(GeometryRefiner self, mfem::Geometry::Type Geom, int Times) -> IntegrationRule"""
        return _geom.GeometryRefiner_RefineInterior(self, Geom, Times)
    RefineInterior = _swig_new_instance_method(_geom.GeometryRefiner_RefineInterior)

    def GetRefinementLevelFromPoints(self, Geom, Npts):
        r"""GetRefinementLevelFromPoints(GeometryRefiner self, mfem::Geometry::Type Geom, int Npts) -> int"""
        return _geom.GeometryRefiner_GetRefinementLevelFromPoints(self, Geom, Npts)
    GetRefinementLevelFromPoints = _swig_new_instance_method(_geom.GeometryRefiner_GetRefinementLevelFromPoints)

    def GetRefinementLevelFromElems(self, geom, Npts):
        r"""GetRefinementLevelFromElems(GeometryRefiner self, mfem::Geometry::Type geom, int Npts) -> int"""
        return _geom.GeometryRefiner_GetRefinementLevelFromElems(self, geom, Npts)
    GetRefinementLevelFromElems = _swig_new_instance_method(_geom.GeometryRefiner_GetRefinementLevelFromElems)
    __swig_destroy__ = _geom.delete_GeometryRefiner

# Register GeometryRefiner in _geom:
_geom.GeometryRefiner_swigregister(GeometryRefiner)

class GeometryTypeArray(object):
    r"""Proxy of C++ mfem::Array< mfem::Geometry::Type > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(GeometryTypeArray self) -> GeometryTypeArray
        __init__(GeometryTypeArray self, mfem::MemoryType mt) -> GeometryTypeArray
        __init__(GeometryTypeArray self, int asize) -> GeometryTypeArray
        __init__(GeometryTypeArray self, mfem::Geometry::Type * _data, int asize) -> GeometryTypeArray
        __init__(GeometryTypeArray self, GeometryTypeArray src) -> GeometryTypeArray
        """
        _geom.GeometryTypeArray_swiginit(self, _geom.new_GeometryTypeArray(*args))

        if len(args) == 1 and isinstance(args[0], list):
            self.MakeDataOwner()



    __swig_destroy__ = _geom.delete_GeometryTypeArray

    def GetData(self, *args):
        r"""
        GetData(GeometryTypeArray self) -> mfem::Geometry::Type
        GetData(GeometryTypeArray self) -> mfem::Geometry::Type const *
        """
        return _geom.GeometryTypeArray_GetData(self, *args)
    GetData = _swig_new_instance_method(_geom.GeometryTypeArray_GetData)

    def GetMemory(self, *args):
        r"""
        GetMemory(GeometryTypeArray self) -> mfem::Memory< mfem::Geometry::Type >
        GetMemory(GeometryTypeArray self) -> mfem::Memory< mfem::Geometry::Type > const &
        """
        return _geom.GeometryTypeArray_GetMemory(self, *args)
    GetMemory = _swig_new_instance_method(_geom.GeometryTypeArray_GetMemory)

    def UseDevice(self):
        r"""UseDevice(GeometryTypeArray self) -> bool"""
        return _geom.GeometryTypeArray_UseDevice(self)
    UseDevice = _swig_new_instance_method(_geom.GeometryTypeArray_UseDevice)

    def OwnsData(self):
        r"""OwnsData(GeometryTypeArray self) -> bool"""
        return _geom.GeometryTypeArray_OwnsData(self)
    OwnsData = _swig_new_instance_method(_geom.GeometryTypeArray_OwnsData)

    def StealData(self, p):
        r"""StealData(GeometryTypeArray self, mfem::Geometry::Type ** p)"""
        return _geom.GeometryTypeArray_StealData(self, p)
    StealData = _swig_new_instance_method(_geom.GeometryTypeArray_StealData)

    def LoseData(self):
        r"""LoseData(GeometryTypeArray self)"""
        return _geom.GeometryTypeArray_LoseData(self)
    LoseData = _swig_new_instance_method(_geom.GeometryTypeArray_LoseData)

    def MakeDataOwner(self):
        r"""MakeDataOwner(GeometryTypeArray self)"""
        return _geom.GeometryTypeArray_MakeDataOwner(self)
    MakeDataOwner = _swig_new_instance_method(_geom.GeometryTypeArray_MakeDataOwner)

    def Size(self):
        r"""Size(GeometryTypeArray self) -> int"""
        return _geom.GeometryTypeArray_Size(self)
    Size = _swig_new_instance_method(_geom.GeometryTypeArray_Size)

    def SetSize(self, *args):
        r"""
        SetSize(GeometryTypeArray self, int nsize)
        SetSize(GeometryTypeArray self, int nsize, mfem::Geometry::Type const & initval)
        SetSize(GeometryTypeArray self, int nsize, mfem::MemoryType mt)
        """
        return _geom.GeometryTypeArray_SetSize(self, *args)
    SetSize = _swig_new_instance_method(_geom.GeometryTypeArray_SetSize)

    def Capacity(self):
        r"""Capacity(GeometryTypeArray self) -> int"""
        return _geom.GeometryTypeArray_Capacity(self)
    Capacity = _swig_new_instance_method(_geom.GeometryTypeArray_Capacity)

    def Reserve(self, capacity):
        r"""Reserve(GeometryTypeArray self, int capacity)"""
        return _geom.GeometryTypeArray_Reserve(self, capacity)
    Reserve = _swig_new_instance_method(_geom.GeometryTypeArray_Reserve)

    def Append(self, *args):
        r"""
        Append(GeometryTypeArray self, mfem::Geometry::Type const & el) -> int
        Append(GeometryTypeArray self, mfem::Geometry::Type const * els, int nels) -> int
        Append(GeometryTypeArray self, GeometryTypeArray els) -> int
        """
        return _geom.GeometryTypeArray_Append(self, *args)
    Append = _swig_new_instance_method(_geom.GeometryTypeArray_Append)

    def Prepend(self, el):
        r"""Prepend(GeometryTypeArray self, mfem::Geometry::Type const & el) -> int"""
        return _geom.GeometryTypeArray_Prepend(self, el)
    Prepend = _swig_new_instance_method(_geom.GeometryTypeArray_Prepend)

    def Last(self, *args):
        r"""
        Last(GeometryTypeArray self) -> mfem::Geometry::Type
        Last(GeometryTypeArray self) -> mfem::Geometry::Type const &
        """
        return _geom.GeometryTypeArray_Last(self, *args)
    Last = _swig_new_instance_method(_geom.GeometryTypeArray_Last)

    def DeleteLast(self):
        r"""DeleteLast(GeometryTypeArray self)"""
        return _geom.GeometryTypeArray_DeleteLast(self)
    DeleteLast = _swig_new_instance_method(_geom.GeometryTypeArray_DeleteLast)

    def DeleteAll(self):
        r"""DeleteAll(GeometryTypeArray self)"""
        return _geom.GeometryTypeArray_DeleteAll(self)
    DeleteAll = _swig_new_instance_method(_geom.GeometryTypeArray_DeleteAll)

    def Copy(self, copy):
        r"""Copy(GeometryTypeArray self, GeometryTypeArray copy)"""
        return _geom.GeometryTypeArray_Copy(self, copy)
    Copy = _swig_new_instance_method(_geom.GeometryTypeArray_Copy)

    def MakeRef(self, *args):
        r"""
        MakeRef(GeometryTypeArray self, mfem::Geometry::Type * arg2, int arg3)
        MakeRef(GeometryTypeArray self, GeometryTypeArray master)
        """
        return _geom.GeometryTypeArray_MakeRef(self, *args)
    MakeRef = _swig_new_instance_method(_geom.GeometryTypeArray_MakeRef)

    def GetSubArray(self, offset, sa_size, sa):
        r"""GetSubArray(GeometryTypeArray self, int offset, int sa_size, GeometryTypeArray sa)"""
        return _geom.GeometryTypeArray_GetSubArray(self, offset, sa_size, sa)
    GetSubArray = _swig_new_instance_method(_geom.GeometryTypeArray_GetSubArray)

    def begin(self, *args):
        r"""
        begin(GeometryTypeArray self) -> mfem::Geometry::Type
        begin(GeometryTypeArray self) -> mfem::Geometry::Type const *
        """
        return _geom.GeometryTypeArray_begin(self, *args)
    begin = _swig_new_instance_method(_geom.GeometryTypeArray_begin)

    def end(self, *args):
        r"""
        end(GeometryTypeArray self) -> mfem::Geometry::Type
        end(GeometryTypeArray self) -> mfem::Geometry::Type const *
        """
        return _geom.GeometryTypeArray_end(self, *args)
    end = _swig_new_instance_method(_geom.GeometryTypeArray_end)

    def MemoryUsage(self):
        r"""MemoryUsage(GeometryTypeArray self) -> long"""
        return _geom.GeometryTypeArray_MemoryUsage(self)
    MemoryUsage = _swig_new_instance_method(_geom.GeometryTypeArray_MemoryUsage)

    def Read(self, on_dev=True):
        r"""Read(GeometryTypeArray self, bool on_dev=True) -> mfem::Geometry::Type const *"""
        return _geom.GeometryTypeArray_Read(self, on_dev)
    Read = _swig_new_instance_method(_geom.GeometryTypeArray_Read)

    def HostRead(self):
        r"""HostRead(GeometryTypeArray self) -> mfem::Geometry::Type const *"""
        return _geom.GeometryTypeArray_HostRead(self)
    HostRead = _swig_new_instance_method(_geom.GeometryTypeArray_HostRead)

    def Write(self, on_dev=True):
        r"""Write(GeometryTypeArray self, bool on_dev=True) -> mfem::Geometry::Type *"""
        return _geom.GeometryTypeArray_Write(self, on_dev)
    Write = _swig_new_instance_method(_geom.GeometryTypeArray_Write)

    def HostWrite(self):
        r"""HostWrite(GeometryTypeArray self) -> mfem::Geometry::Type *"""
        return _geom.GeometryTypeArray_HostWrite(self)
    HostWrite = _swig_new_instance_method(_geom.GeometryTypeArray_HostWrite)

    def ReadWrite(self, on_dev=True):
        r"""ReadWrite(GeometryTypeArray self, bool on_dev=True) -> mfem::Geometry::Type *"""
        return _geom.GeometryTypeArray_ReadWrite(self, on_dev)
    ReadWrite = _swig_new_instance_method(_geom.GeometryTypeArray_ReadWrite)

    def HostReadWrite(self):
        r"""HostReadWrite(GeometryTypeArray self) -> mfem::Geometry::Type *"""
        return _geom.GeometryTypeArray_HostReadWrite(self)
    HostReadWrite = _swig_new_instance_method(_geom.GeometryTypeArray_HostReadWrite)

    def __setitem__(self, i, v):
        r"""__setitem__(GeometryTypeArray self, int i, mfem::Geometry::Type const v)"""
        return _geom.GeometryTypeArray___setitem__(self, i, v)
    __setitem__ = _swig_new_instance_method(_geom.GeometryTypeArray___setitem__)

    def __getitem__(self, i):
        r"""__getitem__(GeometryTypeArray self, int const i) -> mfem::Geometry::Type const &"""
        return _geom.GeometryTypeArray___getitem__(self, i)
    __getitem__ = _swig_new_instance_method(_geom.GeometryTypeArray___getitem__)

    def Assign(self, *args):
        r"""
        Assign(GeometryTypeArray self, mfem::Geometry::Type const * arg2)
        Assign(GeometryTypeArray self, mfem::Geometry::Type const & a)
        """
        return _geom.GeometryTypeArray_Assign(self, *args)
    Assign = _swig_new_instance_method(_geom.GeometryTypeArray_Assign)

    def ToList(self):
        return [self[i] for i in range(self.Size())]



# Register GeometryTypeArray in _geom:
_geom.GeometryTypeArray_swigregister(GeometryTypeArray)
GlobGeometryRefiner = cvar.GlobGeometryRefiner






import sys

if len(sys.argv) > 1 and sys.argv[1] == '-p':
    import mfem.par as mfem
    use_parallel = True

    from mfem.common.mpi_debug import nicePrint as print

    from mpi4py import MPI
    myid = MPI.COMM_WORLD.rank

else:
    import mfem.ser as mfem
    use_parallel = False
    myid = 0

def test_course_to_file_map():
    mesh_file = "../data/inline-quad.mesh"
    device = mfem.Device('cpu')

    mesh = mfem.Mesh(mesh_file, 1, 1)

    refinements = mfem.RefinementArray()
    refinements.Append(mfem.Refinement(0, 0b11))
    refinements.Append(mfem.Refinement(1, 0b10))
    refinements.Append(mfem.Refinement(2, 0b01))

    mesh.GeneralRefinement(refinements)

    cft = mesh.GetRefinementTransforms()

    coarse_to_fine = mfem.Table()
    coarse_to_ref_type = mfem.intArray()
    ref_type_to_matrix = mfem.Table()
    ref_type_to_geom = mfem.GeometryTypeArray()

    cft.GetCoarseToFineMap(mesh,
                           coarse_to_fine,
                           coarse_to_ref_type,
                           ref_type_to_matrix,
                           ref_type_to_geom)
    print("coarse_to_fine element number mapping:")
    coarse_to_fine.Print()

    print("ref_type_to_geom mapping:")

    for i in range(ref_type_to_geom.Size()):
        g = ref_type_to_geom[i]
        print("ref_type: " + str(i) + "=> geom: " + str(g))

if __name__ == '__main__':
    test_course_to_file_map()

'''
#include "mfem.hpp"
#include <fstream>
#include <iostream>

using namespace std;
using namespace mfem;

int main(int argc, char *argv[])
{
   const char *mesh_file = "../data/inline-quad.mesh";

   const char *device_config = "cpu";
   Device device(device_config);

   Mesh mesh(mesh_file, 1, 1);
   int dim = mesh.Dimension();

   Array<Refinement> refinements;
   refinements.Append(Refinement(0,0b11));
   refinements.Append(Refinement(1,0b10));
   refinements.Append(Refinement(2,0b01));
 
   mesh.GeneralRefinement(refinements);

   mfem::CoarseFineTransformations const & coarse_fine_transform
      = mesh.GetRefinementTransforms();
   mfem::Table coarse_to_fine;
   mfem::Array<int> coarse_to_ref_type;
   mfem::Table ref_type_to_matrix;
   mfem::Array<mfem::Geometry::Type> ref_type_to_geom;
   coarse_fine_transform.GetCoarseToFineMap(mesh, coarse_to_fine,
                                            coarse_to_ref_type,
                                            ref_type_to_matrix,
                                            ref_type_to_geom);

   printf("coarse_to_fine element number mapping:\n");
   coarse_to_fine.Print();

   printf("ref_type_to_geom mapping:\n");
   for (int i = 0; i < ref_type_to_geom.Size(); i++) {
      Geometry::Type g = ref_type_to_geom[i];
      printf("ref_type: %d => geom: %d\n",i,g);
   }
}
'''



from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import
'''

Test for PyMFEM

  This script runs C++ version and Python version examples
  and compare numbers in output.

  Usage
    usage: test.py [-h] [-np NP] [-verbose] [-parallel] [-serial]
               [-clean] [-ex EX] [-mfempdir '../../mfem']
               [-mfemsdir '../../mfem_ser']
    optional arguments:
      -h, --help          show this help message and exit
      -np NP              Number of processes for MPI
      -verbose            Verbose print out
      -parallel           Test parallel version
      -serial             Test serial version
      -clean              clean temporary files
      -ex EX              Test one example  python test.py -verbose
      -mfempdir           Parallel MFEM build directory (examples folder should exist)
      -mfemsdir           Serial MFEM build directory (examples folder should exist)

  Note:

  It compares numbers appears in the last 5 lines
  of output text, which seems sufficient.

  Number of precision is not the same between C++ and Python,
  the script only compare the first 3 significant digits.

  It ignores CPU seconds in output.

  

'''
import os
import sys
import subprocess as sp
import time
import shutil
import re

skip_test = [''] 
#skip_test = ['ex15p']  #skip this since it takes too long on my Mac..;D

class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'

## special handling of text (needs to be set manually)
ignore_txt_def = ['seconds', 'residual']    
ignore_txt_dict = {'ex15p':['seconds', 'residual',],
                   'ex10':['iteration', 'seconds', 'residual',],
                   'ex10p':['iteration', 'seconds', 'residual',],
                   'ex13p':['iteration', 'seconds', 'residual',]}


def run_file(command, num = 5):
    t1 = time.time()
    p = sp.Popen(command,stdout=sp.PIPE, stderr=sp.STDOUT, stdin=sp.PIPE)
    p.stdin.write(b'q\n')
    lines, errs = p.communicate()
    lines = lines.decode('utf-8').split('\n')
    t2 = time.time()
    sys.stdout.flush()
    lines = [l for l in lines if len(l.strip()) != 0]
    return t2-t1, lines[-num:]

def pad_lines(lines):
    length = max([len(l.strip()) for l in lines])
    return  [l.strip() + " "*(length-len(l.strip())) for l in lines]

def extract_numbers(line, case):
    nums = []
    line = line.strip()

    igt = ignore_txt_dict.get(case, ignore_txt_def)
    for x in igt: 
        if line.lower().find(x) != -1: return []
    for x in line.split():
        try:
            if x.endswith(','): x = x[:-1]
            if x.endswith('.'): x = x[:-1]            
            x = float(x)
            nums.append("{:.3e}".format(x))
        except:
            pass
    return nums

def compare_results(lines1, lines2, case, verbose=True):
    lines1 = pad_lines(lines1)
    lines2 = pad_lines(lines2)

    compare = []
    for l1, l2 in zip(lines1, lines2):
        nums1 = extract_numbers(l1, case)
        nums2 = extract_numbers(l2, case)
        flag = nums1 == nums2
        if len(nums1) == 0 and len(nums2) == 0: flag = 'ignored'
        if verbose:print(l1 + "\t" + l2 + " : " + str(flag))
        if flag == 'ignored': continue
        compare.append(flag)
    return all(compare)
    
def run_test(mfem_exes, pymfem_exes, serial=True, np=2, verbose=False):
    print("mfem examples from : " + os.path.dirname(mfem_exes[0]))
    print("PyMFEM examples from : " + os.path.dirname(pymfem_exes[0]))
    print(',  '.join([os.path.basename(x) for x in mfem_exes]))    

    if not serial:
        comm_mpi = ["mpirun", "-np", str(np)]
    else:
        comm_mpi = []

    results = []; fails = []

    for e1, e2 in zip(mfem_exes, pymfem_exes):
        case =  os.path.basename(e1)
        print("Running : " + case)
        comm = comm_mpi + [e1]
        t1, l1 = run_file(comm, num = 5)

        print("Running : " + os.path.basename(e2))

        # note -u is unbuffered option
        comm = comm_mpi + [sys.executable,  "-u", e2]
        t2, l2 = run_file(comm, num = 5)

        flag = compare_results(l1, l2, case, verbose=verbose)
        if flag:
            print(os.path.basename(e1) + "  " +  bcolors.OKGREEN + "PASS" + bcolors.ENDC)
        else:
            print(os.path.basename(e1) + "  " +  bcolors.FAIL + "FAIL" + bcolors.ENDC)
            fails.append(os.path.basename(e1))
        results.append(flag)
        print("Time spent for execution : (C) " + str(t1) + " (Python) " + str(t2) + " " + str((t2/t1-1)*100) + "%")
        
    return results, fails
        
def find_mfem_examples(dir, serial = True, example='all'):
    if dir == '': return []
    example_dir = os.path.join(dir, 'examples')
    names = [x for x in os.listdir(example_dir)
             if x.startswith('ex') and not x.endswith('.cpp')]

    if serial:
        names = [x for x in names if not x.endswith('p')]
    else:    
        names = [x for x in names if x.endswith('p')]

    if example != 'all':
        names = [n for n in names if n == example]
    names = [n for n in names if not n in skip_test]    

    nums = [int(''.join(re.findall(r'\d+', n))) for n in names]
    names = [os.path.join(example_dir, x) for x in names]    
    names = [x for x in names if os.access(x, os.X_OK)]

    pairs = [(num, name) for num, name in zip(nums, names)]
    names = [name  for num, name in sorted(pairs)]

    return names

def print_help(self):
    print("python test.py -np -v")

if __name__=="__main__":
    script_path = sys.path[0]  # the location of this file
    
    from mfem.common.arg_parser import ArgParser
    parser = ArgParser(description='test')
    parser.add_argument('-np', default = 2,
                        action = 'store', type = int,
                    help='Number of processes for MPI')
    parser.add_argument('-verbose', 
                        action = 'store_true', default = False,
                        help='Verbose print out')
    parser.add_argument('-parallel', 
                        action = 'store_true', 
                        help='Test parallel version')
    parser.add_argument('-serial', 
                        action = 'store_true', 
                        help='Test serial version')
    parser.add_argument('-clean', 
                        action = 'store_true', default = False,
                        help='Test serial version')
    parser.add_argument('-ex', 
                        action = 'store', default = "all",
                        help='Test one example')
    parser.add_argument('-mfempdir', 
                        action = 'store', default = "../../mfem",
                        help='mfem (parallel) directory')
    parser.add_argument('-mfemsdir', 
                        action = 'store', default = "../../mfem_ser",
                        help='mfem (serial) directory')
    
    
    
    args = parser.parse_args()
    parser.print_options(args)

    np = args.np
    verbose = args.verbose
    testp = bool(args.parallel)
    tests = bool(args.serial)
    clean = args.clean
    example = args.ex
    mfempdir = args.mfempdir
    mfemsdir = args.mfemsdir    


    if clean:
        od = os.getcwd()
        files = os.listdir(od)
        for x in files:
           if x.endswith(".py"): continue
           if os.path.isdir(x):
               shutil.rmtree(x)
           else:
               os.remove(x)
        sys.exit()
    import mfem

    dir1 = os.path.dirname(script_path)
    dir2 = os.path.dirname(os.path.dirname(mfem.__file__))

    print(bcolors.FAIL+'PyMFEM module loaded from :' + dir2 + bcolors.ENDC)
    print(bcolors.FAIL+'Searching Python Example in :' + dir1 + bcolors.ENDC)

    exdir = os.path.join(dir1, "examples")
    
    mfem_dir = os.path.abspath(mfempdir)
    mfemser_dir = os.path.abspath(mfemsdir)

    if testp:
       mfemp_exes = find_mfem_examples(mfem_dir, serial = False, example = example)
       pymfemp_exes = [os.path.join(exdir, os.path.basename(x))+".py"  for x in mfemp_exes]
    if tests:
       mfems_exes = find_mfem_examples(mfemser_dir, serial = True,  example = example)    
       pymfems_exes = [os.path.join(exdir, os.path.basename(x))+".py"  for x in mfems_exes]


    if tests and mfems_exes:
        print(bcolors.BOLD+"Running Serial Version"+bcolors.ENDC)
        results, fails = run_test(mfems_exes, pymfems_exes, serial = True, verbose=verbose)
    else:
        results = []; fails=[]        
    if testp and len(mfemp_exes) != 0:
        print(bcolors.BOLD+"Running Parallel Version"+bcolors.ENDC)
        resultp, failp = run_test(mfemp_exes, pymfemp_exes, serial = False, np = np, verbose=verbose)
    else:
        resultp = []; failp=[]


    if tests and len(results) != 0:
        if all(results):
            print("Serial Test \t" + bcolors.OKGREEN + "ALL PASS" + bcolors.ENDC)
        else:
            print("Serial Test \t" + bcolors.FAIL + " ".join(fails) + bcolors.ENDC)

    if testp and len(resultp) != 0:
        if all(resultp):
            print("Parallel Test \t" + bcolors.OKGREEN + "ALL PASS" + bcolors.ENDC)
        else:
            print("Parallel Test \t" + bcolors.FAIL + " ".join(failp) + bcolors.ENDC)        
        
    
             
    
    
    
    



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _prestriction
else:
    import _prestriction

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _prestriction.SWIG_PyInstanceMethod_New
_swig_new_static_method = _prestriction.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._par.element
import mfem._par.globals
import mfem._par.array
import mfem._par.mem_manager
import mfem._par.densemat
import mfem._par.vector
import mfem._par.operators
import mfem._par.matrix
import mfem._par.geom
import mfem._par.intrules
import mfem._par.table
import mfem._par.hash
import mfem._par.restriction
import mfem._par.mesh
import mfem._par.sort_pairs
import mfem._par.ncmesh
import mfem._par.vtk
import mfem._par.vertex
import mfem._par.gridfunc
import mfem._par.coefficient
import mfem._par.sparsemat
import mfem._par.eltrans
import mfem._par.fe
import mfem._par.fespace
import mfem._par.fe_coll
import mfem._par.lininteg
import mfem._par.handle
import mfem._par.hypre
import mfem._par.bilininteg
import mfem._par.linearform
class ParL2FaceRestriction(mfem._par.restriction.L2FaceRestriction):
    r"""Proxy of C++ mfem::ParL2FaceRestriction class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args, **kwargs):
        r"""__init__(ParL2FaceRestriction self, mfem::ParFiniteElementSpace const & arg2, mfem::ElementDofOrdering arg3, mfem::FaceType type, mfem::L2FaceValues m=DoubleValued) -> ParL2FaceRestriction"""
        _prestriction.ParL2FaceRestriction_swiginit(self, _prestriction.new_ParL2FaceRestriction(*args, **kwargs))

    def Mult(self, x, y):
        r"""Mult(ParL2FaceRestriction self, Vector x, Vector y)"""
        return _prestriction.ParL2FaceRestriction_Mult(self, x, y)
    Mult = _swig_new_instance_method(_prestriction.ParL2FaceRestriction_Mult)

    def FillI(self, mat, face_mat):
        r"""FillI(ParL2FaceRestriction self, SparseMatrix mat, SparseMatrix face_mat)"""
        return _prestriction.ParL2FaceRestriction_FillI(self, mat, face_mat)
    FillI = _swig_new_instance_method(_prestriction.ParL2FaceRestriction_FillI)

    def FillJAndData(self, ea_data, mat, face_mat):
        r"""FillJAndData(ParL2FaceRestriction self, Vector ea_data, SparseMatrix mat, SparseMatrix face_mat)"""
        return _prestriction.ParL2FaceRestriction_FillJAndData(self, ea_data, mat, face_mat)
    FillJAndData = _swig_new_instance_method(_prestriction.ParL2FaceRestriction_FillJAndData)
    __swig_destroy__ = _prestriction.delete_ParL2FaceRestriction

# Register ParL2FaceRestriction in _prestriction:
_prestriction.ParL2FaceRestriction_swigregister(ParL2FaceRestriction)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _blockvector
else:
    import _blockvector

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _blockvector.SWIG_PyInstanceMethod_New
_swig_new_static_method = _blockvector.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import mfem._ser.array
import mfem._ser.mem_manager
import mfem._ser.vector
class BlockVector(mfem._ser.vector.Vector):
    r"""Proxy of C++ mfem::BlockVector class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(BlockVector self) -> BlockVector
        __init__(BlockVector self, intArray bOffsets) -> BlockVector
        __init__(BlockVector self, intArray bOffsets, mfem::MemoryType mt) -> BlockVector
        __init__(BlockVector self, BlockVector block) -> BlockVector
        __init__(BlockVector self, double * data, intArray bOffsets) -> BlockVector
        """
        _blockvector.BlockVector_swiginit(self, _blockvector.new_BlockVector(*args))

    def NumBlocks(self):
        r"""NumBlocks(BlockVector self) -> int"""
        return _blockvector.BlockVector_NumBlocks(self)
    NumBlocks = _swig_new_instance_method(_blockvector.BlockVector_NumBlocks)
    __swig_destroy__ = _blockvector.delete_BlockVector

    def GetBlock(self, *args):
        r"""
        GetBlock(BlockVector self, int i) -> Vector
        GetBlock(BlockVector self, int i) -> Vector
        """
        return _blockvector.BlockVector_GetBlock(self, *args)
    GetBlock = _swig_new_instance_method(_blockvector.BlockVector_GetBlock)

    def GetBlockView(self, i, blockView):
        r"""GetBlockView(BlockVector self, int i, Vector blockView)"""
        return _blockvector.BlockVector_GetBlockView(self, i, blockView)
    GetBlockView = _swig_new_instance_method(_blockvector.BlockVector_GetBlockView)

    def BlockSize(self, i):
        r"""BlockSize(BlockVector self, int i) -> int"""
        return _blockvector.BlockVector_BlockSize(self, i)
    BlockSize = _swig_new_instance_method(_blockvector.BlockVector_BlockSize)

    def Update(self, *args):
        r"""
        Update(BlockVector self, double * data, intArray bOffsets)
        Update(BlockVector self, Vector data, intArray bOffsets)
        Update(BlockVector self, intArray bOffsets)
        Update(BlockVector self, intArray bOffsets, mfem::MemoryType mt)
        """
        return _blockvector.BlockVector_Update(self, *args)
    Update = _swig_new_instance_method(_blockvector.BlockVector_Update)

# Register BlockVector in _blockvector:
_blockvector.BlockVector_swigregister(BlockVector)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _ncmesh
else:
    import _ncmesh

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _ncmesh.SWIG_PyInstanceMethod_New
_swig_new_static_method = _ncmesh.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._par.mesh
import mfem._par.matrix
import mfem._par.vector
import mfem._par.array
import mfem._par.mem_manager
import mfem._par.operators
import mfem._par.sort_pairs
import mfem._par.vtk
import mfem._par.element
import mfem._par.globals
import mfem._par.densemat
import mfem._par.geom
import mfem._par.intrules
import mfem._par.table
import mfem._par.hash
import mfem._par.vertex
import mfem._par.gridfunc
import mfem._par.coefficient
import mfem._par.sparsemat
import mfem._par.eltrans
import mfem._par.fe
import mfem._par.fespace
import mfem._par.fe_coll
import mfem._par.lininteg
import mfem._par.handle
import mfem._par.hypre
import mfem._par.restriction
import mfem._par.bilininteg
import mfem._par.linearform
class Refinement(object):
    r"""Proxy of C++ mfem::Refinement class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    index = property(_ncmesh.Refinement_index_get, _ncmesh.Refinement_index_set, doc=r"""index : int""")
    ref_type = property(_ncmesh.Refinement_ref_type_get, _ncmesh.Refinement_ref_type_set, doc=r"""ref_type : char""")

    def __init__(self, *args):
        r"""
        __init__(Refinement self) -> Refinement
        __init__(Refinement self, int index, int type=7) -> Refinement
        """
        _ncmesh.Refinement_swiginit(self, _ncmesh.new_Refinement(*args))
    __swig_destroy__ = _ncmesh.delete_Refinement

# Register Refinement in _ncmesh:
_ncmesh.Refinement_swigregister(Refinement)

class Embedding(object):
    r"""Proxy of C++ mfem::Embedding class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    parent = property(_ncmesh.Embedding_parent_get, _ncmesh.Embedding_parent_set, doc=r"""parent : int""")
    matrix = property(_ncmesh.Embedding_matrix_get, _ncmesh.Embedding_matrix_set, doc=r"""matrix : int""")

    def __init__(self, *args):
        r"""
        __init__(Embedding self) -> Embedding
        __init__(Embedding self, int elem, int matrix=0) -> Embedding
        """
        _ncmesh.Embedding_swiginit(self, _ncmesh.new_Embedding(*args))
    __swig_destroy__ = _ncmesh.delete_Embedding

# Register Embedding in _ncmesh:
_ncmesh.Embedding_swigregister(Embedding)

class CoarseFineTransformations(object):
    r"""Proxy of C++ mfem::CoarseFineTransformations class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    point_matrices = property(_ncmesh.CoarseFineTransformations_point_matrices_get, _ncmesh.CoarseFineTransformations_point_matrices_set, doc=r"""point_matrices : a(mfem::Geometry::NumGeom).mfem::DenseTensor""")
    embeddings = property(_ncmesh.CoarseFineTransformations_embeddings_get, doc=r"""embeddings : mfem::Array<(mfem::Embedding)>""")

    def GetCoarseToFineMap(self, fine_mesh, coarse_to_fine, coarse_to_ref_type, ref_type_to_matrix, ref_type_to_geom):
        r"""GetCoarseToFineMap(CoarseFineTransformations self, Mesh fine_mesh, Table coarse_to_fine, intArray coarse_to_ref_type, Table ref_type_to_matrix, GeometryTypeArray ref_type_to_geom)"""
        return _ncmesh.CoarseFineTransformations_GetCoarseToFineMap(self, fine_mesh, coarse_to_fine, coarse_to_ref_type, ref_type_to_matrix, ref_type_to_geom)
    GetCoarseToFineMap = _swig_new_instance_method(_ncmesh.CoarseFineTransformations_GetCoarseToFineMap)

    def Clear(self):
        r"""Clear(CoarseFineTransformations self)"""
        return _ncmesh.CoarseFineTransformations_Clear(self)
    Clear = _swig_new_instance_method(_ncmesh.CoarseFineTransformations_Clear)

    def IsInitialized(self):
        r"""IsInitialized(CoarseFineTransformations self) -> bool"""
        return _ncmesh.CoarseFineTransformations_IsInitialized(self)
    IsInitialized = _swig_new_instance_method(_ncmesh.CoarseFineTransformations_IsInitialized)

    def MemoryUsage(self):
        r"""MemoryUsage(CoarseFineTransformations self) -> long"""
        return _ncmesh.CoarseFineTransformations_MemoryUsage(self)
    MemoryUsage = _swig_new_instance_method(_ncmesh.CoarseFineTransformations_MemoryUsage)

    def __init__(self):
        r"""__init__(CoarseFineTransformations self) -> CoarseFineTransformations"""
        _ncmesh.CoarseFineTransformations_swiginit(self, _ncmesh.new_CoarseFineTransformations())
    __swig_destroy__ = _ncmesh.delete_CoarseFineTransformations

# Register CoarseFineTransformations in _ncmesh:
_ncmesh.CoarseFineTransformations_swigregister(CoarseFineTransformations)

class NCMesh(object):
    r"""Proxy of C++ mfem::NCMesh class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(NCMesh self, Mesh mesh, std::istream * vertex_parents=None) -> NCMesh
        __init__(NCMesh self, NCMesh other) -> NCMesh
        """
        _ncmesh.NCMesh_swiginit(self, _ncmesh.new_NCMesh(*args))
    __swig_destroy__ = _ncmesh.delete_NCMesh

    def Dimension(self):
        r"""Dimension(NCMesh self) -> int"""
        return _ncmesh.NCMesh_Dimension(self)
    Dimension = _swig_new_instance_method(_ncmesh.NCMesh_Dimension)

    def SpaceDimension(self):
        r"""SpaceDimension(NCMesh self) -> int"""
        return _ncmesh.NCMesh_SpaceDimension(self)
    SpaceDimension = _swig_new_instance_method(_ncmesh.NCMesh_SpaceDimension)

    def GetNVertices(self):
        r"""GetNVertices(NCMesh self) -> int"""
        return _ncmesh.NCMesh_GetNVertices(self)
    GetNVertices = _swig_new_instance_method(_ncmesh.NCMesh_GetNVertices)

    def GetNEdges(self):
        r"""GetNEdges(NCMesh self) -> int"""
        return _ncmesh.NCMesh_GetNEdges(self)
    GetNEdges = _swig_new_instance_method(_ncmesh.NCMesh_GetNEdges)

    def GetNFaces(self):
        r"""GetNFaces(NCMesh self) -> int"""
        return _ncmesh.NCMesh_GetNFaces(self)
    GetNFaces = _swig_new_instance_method(_ncmesh.NCMesh_GetNFaces)

    def Refine(self, refinements):
        r"""Refine(NCMesh self, RefinementArray refinements)"""
        return _ncmesh.NCMesh_Refine(self, refinements)
    Refine = _swig_new_instance_method(_ncmesh.NCMesh_Refine)

    def LimitNCLevel(self, max_nc_level):
        r"""LimitNCLevel(NCMesh self, int max_nc_level)"""
        return _ncmesh.NCMesh_LimitNCLevel(self, max_nc_level)
    LimitNCLevel = _swig_new_instance_method(_ncmesh.NCMesh_LimitNCLevel)

    def GetDerefinementTable(self):
        r"""GetDerefinementTable(NCMesh self) -> Table"""
        return _ncmesh.NCMesh_GetDerefinementTable(self)
    GetDerefinementTable = _swig_new_instance_method(_ncmesh.NCMesh_GetDerefinementTable)

    def CheckDerefinementNCLevel(self, deref_table, level_ok, max_nc_level):
        r"""CheckDerefinementNCLevel(NCMesh self, Table deref_table, intArray level_ok, int max_nc_level)"""
        return _ncmesh.NCMesh_CheckDerefinementNCLevel(self, deref_table, level_ok, max_nc_level)
    CheckDerefinementNCLevel = _swig_new_instance_method(_ncmesh.NCMesh_CheckDerefinementNCLevel)

    def Derefine(self, derefs):
        r"""Derefine(NCMesh self, intArray derefs)"""
        return _ncmesh.NCMesh_Derefine(self, derefs)
    Derefine = _swig_new_instance_method(_ncmesh.NCMesh_Derefine)

    def GetFaceList(self):
        r"""GetFaceList(NCMesh self) -> mfem::NCMesh::NCList const &"""
        return _ncmesh.NCMesh_GetFaceList(self)
    GetFaceList = _swig_new_instance_method(_ncmesh.NCMesh_GetFaceList)

    def GetEdgeList(self):
        r"""GetEdgeList(NCMesh self) -> mfem::NCMesh::NCList const &"""
        return _ncmesh.NCMesh_GetEdgeList(self)
    GetEdgeList = _swig_new_instance_method(_ncmesh.NCMesh_GetEdgeList)

    def GetVertexList(self):
        r"""GetVertexList(NCMesh self) -> mfem::NCMesh::NCList const &"""
        return _ncmesh.NCMesh_GetVertexList(self)
    GetVertexList = _swig_new_instance_method(_ncmesh.NCMesh_GetVertexList)

    def GetNCList(self, entity):
        r"""GetNCList(NCMesh self, int entity) -> mfem::NCMesh::NCList const &"""
        return _ncmesh.NCMesh_GetNCList(self, entity)
    GetNCList = _swig_new_instance_method(_ncmesh.NCMesh_GetNCList)

    def MarkCoarseLevel(self):
        r"""MarkCoarseLevel(NCMesh self)"""
        return _ncmesh.NCMesh_MarkCoarseLevel(self)
    MarkCoarseLevel = _swig_new_instance_method(_ncmesh.NCMesh_MarkCoarseLevel)

    def GetRefinementTransforms(self):
        r"""GetRefinementTransforms(NCMesh self) -> CoarseFineTransformations"""
        return _ncmesh.NCMesh_GetRefinementTransforms(self)
    GetRefinementTransforms = _swig_new_instance_method(_ncmesh.NCMesh_GetRefinementTransforms)

    def GetDerefinementTransforms(self):
        r"""GetDerefinementTransforms(NCMesh self) -> CoarseFineTransformations"""
        return _ncmesh.NCMesh_GetDerefinementTransforms(self)
    GetDerefinementTransforms = _swig_new_instance_method(_ncmesh.NCMesh_GetDerefinementTransforms)

    def ClearTransforms(self):
        r"""ClearTransforms(NCMesh self)"""
        return _ncmesh.NCMesh_ClearTransforms(self)
    ClearTransforms = _swig_new_instance_method(_ncmesh.NCMesh_ClearTransforms)

    @staticmethod
    def GridSfcOrdering2D(width, height, coords):
        r"""GridSfcOrdering2D(int width, int height, intArray coords)"""
        return _ncmesh.NCMesh_GridSfcOrdering2D(width, height, coords)
    GridSfcOrdering2D = _swig_new_static_method(_ncmesh.NCMesh_GridSfcOrdering2D)

    @staticmethod
    def GridSfcOrdering3D(width, height, depth, coords):
        r"""GridSfcOrdering3D(int width, int height, int depth, intArray coords)"""
        return _ncmesh.NCMesh_GridSfcOrdering3D(width, height, depth, coords)
    GridSfcOrdering3D = _swig_new_static_method(_ncmesh.NCMesh_GridSfcOrdering3D)

    def GetEdgeVertices(self, edge_id, vert_index, oriented=True):
        r"""GetEdgeVertices(NCMesh self, mfem::NCMesh::MeshId const & edge_id, int [2] vert_index, bool oriented=True)"""
        return _ncmesh.NCMesh_GetEdgeVertices(self, edge_id, vert_index, oriented)
    GetEdgeVertices = _swig_new_instance_method(_ncmesh.NCMesh_GetEdgeVertices)

    def GetEdgeNCOrientation(self, edge_id):
        r"""GetEdgeNCOrientation(NCMesh self, mfem::NCMesh::MeshId const & edge_id) -> int"""
        return _ncmesh.NCMesh_GetEdgeNCOrientation(self, edge_id)
    GetEdgeNCOrientation = _swig_new_instance_method(_ncmesh.NCMesh_GetEdgeNCOrientation)

    def GetFaceVerticesEdges(self, face_id, vert_index, edge_index, edge_orientation):
        r"""GetFaceVerticesEdges(NCMesh self, mfem::NCMesh::MeshId const & face_id, int [4] vert_index, int [4] edge_index, int [4] edge_orientation) -> int"""
        return _ncmesh.NCMesh_GetFaceVerticesEdges(self, face_id, vert_index, edge_index, edge_orientation)
    GetFaceVerticesEdges = _swig_new_instance_method(_ncmesh.NCMesh_GetFaceVerticesEdges)

    def GetEdgeMaster(self, v1, v2):
        r"""GetEdgeMaster(NCMesh self, int v1, int v2) -> int"""
        return _ncmesh.NCMesh_GetEdgeMaster(self, v1, v2)
    GetEdgeMaster = _swig_new_instance_method(_ncmesh.NCMesh_GetEdgeMaster)

    def GetBoundaryClosure(self, bdr_attr_is_ess, bdr_vertices, bdr_edges):
        r"""GetBoundaryClosure(NCMesh self, intArray bdr_attr_is_ess, intArray bdr_vertices, intArray bdr_edges)"""
        return _ncmesh.NCMesh_GetBoundaryClosure(self, bdr_attr_is_ess, bdr_vertices, bdr_edges)
    GetBoundaryClosure = _swig_new_instance_method(_ncmesh.NCMesh_GetBoundaryClosure)

    def GetElementGeometry(self, index):
        r"""GetElementGeometry(NCMesh self, int index) -> mfem::Geometry::Type"""
        return _ncmesh.NCMesh_GetElementGeometry(self, index)
    GetElementGeometry = _swig_new_instance_method(_ncmesh.NCMesh_GetElementGeometry)

    def GetFaceGeometry(self, index):
        r"""GetFaceGeometry(NCMesh self, int index) -> mfem::Geometry::Type"""
        return _ncmesh.NCMesh_GetFaceGeometry(self, index)
    GetFaceGeometry = _swig_new_instance_method(_ncmesh.NCMesh_GetFaceGeometry)

    def GetNumRootElements(self):
        r"""GetNumRootElements(NCMesh self) -> int"""
        return _ncmesh.NCMesh_GetNumRootElements(self)
    GetNumRootElements = _swig_new_instance_method(_ncmesh.NCMesh_GetNumRootElements)

    def GetElementDepth(self, i):
        r"""GetElementDepth(NCMesh self, int i) -> int"""
        return _ncmesh.NCMesh_GetElementDepth(self, i)
    GetElementDepth = _swig_new_instance_method(_ncmesh.NCMesh_GetElementDepth)

    def GetElementSizeReduction(self, i):
        r"""GetElementSizeReduction(NCMesh self, int i) -> int"""
        return _ncmesh.NCMesh_GetElementSizeReduction(self, i)
    GetElementSizeReduction = _swig_new_instance_method(_ncmesh.NCMesh_GetElementSizeReduction)

    def GetElementFacesAttributes(self, i, faces, fattr):
        r"""GetElementFacesAttributes(NCMesh self, int i, intArray faces, intArray fattr)"""
        return _ncmesh.NCMesh_GetElementFacesAttributes(self, i, faces, fattr)
    GetElementFacesAttributes = _swig_new_instance_method(_ncmesh.NCMesh_GetElementFacesAttributes)

    def LoadVertexParents(self, input):
        r"""LoadVertexParents(NCMesh self, std::istream & input)"""
        return _ncmesh.NCMesh_LoadVertexParents(self, input)
    LoadVertexParents = _swig_new_instance_method(_ncmesh.NCMesh_LoadVertexParents)

    def LoadCoarseElements(self, input):
        r"""LoadCoarseElements(NCMesh self, std::istream & input)"""
        return _ncmesh.NCMesh_LoadCoarseElements(self, input)
    LoadCoarseElements = _swig_new_instance_method(_ncmesh.NCMesh_LoadCoarseElements)

    def SetVertexPositions(self, vertices):
        r"""SetVertexPositions(NCMesh self, mfem::Array< mfem::Vertex > const & vertices)"""
        return _ncmesh.NCMesh_SetVertexPositions(self, vertices)
    SetVertexPositions = _swig_new_instance_method(_ncmesh.NCMesh_SetVertexPositions)

    def Trim(self):
        r"""Trim(NCMesh self)"""
        return _ncmesh.NCMesh_Trim(self)
    Trim = _swig_new_instance_method(_ncmesh.NCMesh_Trim)

    def MemoryUsage(self):
        r"""MemoryUsage(NCMesh self) -> long"""
        return _ncmesh.NCMesh_MemoryUsage(self)
    MemoryUsage = _swig_new_instance_method(_ncmesh.NCMesh_MemoryUsage)

    def PrintMemoryDetail(self):
        r"""PrintMemoryDetail(NCMesh self) -> int"""
        return _ncmesh.NCMesh_PrintMemoryDetail(self)
    PrintMemoryDetail = _swig_new_instance_method(_ncmesh.NCMesh_PrintMemoryDetail)

    def PrintVertexParents(self, *args):
        r"""
        PrintVertexParents(NCMesh self, std::ostream & out)
        PrintVertexParents(NCMesh self, char const * file, int precision=8)
        PrintVertexParents(NCMesh self)
        """
        return _ncmesh.NCMesh_PrintVertexParents(self, *args)
    PrintVertexParents = _swig_new_instance_method(_ncmesh.NCMesh_PrintVertexParents)

    def PrintCoarseElements(self, *args):
        r"""
        PrintCoarseElements(NCMesh self, std::ostream & out)
        PrintCoarseElements(NCMesh self, char const * file, int precision=8)
        PrintCoarseElements(NCMesh self)
        """
        return _ncmesh.NCMesh_PrintCoarseElements(self, *args)
    PrintCoarseElements = _swig_new_instance_method(_ncmesh.NCMesh_PrintCoarseElements)

    def PrintStats(self, *args):
        r"""
        PrintStats(NCMesh self, std::ostream & out=out)
        PrintStats(NCMesh self, char const * file, int precision=8)
        """
        return _ncmesh.NCMesh_PrintStats(self, *args)
    PrintStats = _swig_new_instance_method(_ncmesh.NCMesh_PrintStats)

# Register NCMesh in _ncmesh:
_ncmesh.NCMesh_swigregister(NCMesh)

def NCMesh_GridSfcOrdering2D(width, height, coords):
    r"""NCMesh_GridSfcOrdering2D(int width, int height, intArray coords)"""
    return _ncmesh.NCMesh_GridSfcOrdering2D(width, height, coords)
NCMesh_GridSfcOrdering2D = _ncmesh.NCMesh_GridSfcOrdering2D

def NCMesh_GridSfcOrdering3D(width, height, depth, coords):
    r"""NCMesh_GridSfcOrdering3D(int width, int height, int depth, intArray coords)"""
    return _ncmesh.NCMesh_GridSfcOrdering3D(width, height, depth, coords)
NCMesh_GridSfcOrdering3D = _ncmesh.NCMesh_GridSfcOrdering3D

class RefinementArray(object):
    r"""Proxy of C++ mfem::Array< mfem::Refinement > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(RefinementArray self) -> RefinementArray
        __init__(RefinementArray self, mfem::MemoryType mt) -> RefinementArray
        __init__(RefinementArray self, int asize) -> RefinementArray
        __init__(RefinementArray self, Refinement _data, int asize) -> RefinementArray
        __init__(RefinementArray self, RefinementArray src) -> RefinementArray
        """
        _ncmesh.RefinementArray_swiginit(self, _ncmesh.new_RefinementArray(*args))

        if len(args) == 1 and isinstance(args[0], list):
            self.MakeDataOwner()



    __swig_destroy__ = _ncmesh.delete_RefinementArray

    def GetData(self, *args):
        r"""
        GetData(RefinementArray self) -> Refinement
        GetData(RefinementArray self) -> Refinement
        """
        return _ncmesh.RefinementArray_GetData(self, *args)
    GetData = _swig_new_instance_method(_ncmesh.RefinementArray_GetData)

    def GetMemory(self, *args):
        r"""
        GetMemory(RefinementArray self) -> mfem::Memory< mfem::Refinement >
        GetMemory(RefinementArray self) -> mfem::Memory< mfem::Refinement > const &
        """
        return _ncmesh.RefinementArray_GetMemory(self, *args)
    GetMemory = _swig_new_instance_method(_ncmesh.RefinementArray_GetMemory)

    def UseDevice(self):
        r"""UseDevice(RefinementArray self) -> bool"""
        return _ncmesh.RefinementArray_UseDevice(self)
    UseDevice = _swig_new_instance_method(_ncmesh.RefinementArray_UseDevice)

    def OwnsData(self):
        r"""OwnsData(RefinementArray self) -> bool"""
        return _ncmesh.RefinementArray_OwnsData(self)
    OwnsData = _swig_new_instance_method(_ncmesh.RefinementArray_OwnsData)

    def StealData(self, p):
        r"""StealData(RefinementArray self, mfem::Refinement ** p)"""
        return _ncmesh.RefinementArray_StealData(self, p)
    StealData = _swig_new_instance_method(_ncmesh.RefinementArray_StealData)

    def LoseData(self):
        r"""LoseData(RefinementArray self)"""
        return _ncmesh.RefinementArray_LoseData(self)
    LoseData = _swig_new_instance_method(_ncmesh.RefinementArray_LoseData)

    def MakeDataOwner(self):
        r"""MakeDataOwner(RefinementArray self)"""
        return _ncmesh.RefinementArray_MakeDataOwner(self)
    MakeDataOwner = _swig_new_instance_method(_ncmesh.RefinementArray_MakeDataOwner)

    def Size(self):
        r"""Size(RefinementArray self) -> int"""
        return _ncmesh.RefinementArray_Size(self)
    Size = _swig_new_instance_method(_ncmesh.RefinementArray_Size)

    def SetSize(self, *args):
        r"""
        SetSize(RefinementArray self, int nsize)
        SetSize(RefinementArray self, int nsize, Refinement initval)
        SetSize(RefinementArray self, int nsize, mfem::MemoryType mt)
        """
        return _ncmesh.RefinementArray_SetSize(self, *args)
    SetSize = _swig_new_instance_method(_ncmesh.RefinementArray_SetSize)

    def Capacity(self):
        r"""Capacity(RefinementArray self) -> int"""
        return _ncmesh.RefinementArray_Capacity(self)
    Capacity = _swig_new_instance_method(_ncmesh.RefinementArray_Capacity)

    def Reserve(self, capacity):
        r"""Reserve(RefinementArray self, int capacity)"""
        return _ncmesh.RefinementArray_Reserve(self, capacity)
    Reserve = _swig_new_instance_method(_ncmesh.RefinementArray_Reserve)

    def Append(self, *args):
        r"""
        Append(RefinementArray self, Refinement el) -> int
        Append(RefinementArray self, Refinement els, int nels) -> int
        Append(RefinementArray self, RefinementArray els) -> int
        """
        return _ncmesh.RefinementArray_Append(self, *args)
    Append = _swig_new_instance_method(_ncmesh.RefinementArray_Append)

    def Prepend(self, el):
        r"""Prepend(RefinementArray self, Refinement el) -> int"""
        return _ncmesh.RefinementArray_Prepend(self, el)
    Prepend = _swig_new_instance_method(_ncmesh.RefinementArray_Prepend)

    def Last(self, *args):
        r"""
        Last(RefinementArray self) -> Refinement
        Last(RefinementArray self) -> Refinement
        """
        return _ncmesh.RefinementArray_Last(self, *args)
    Last = _swig_new_instance_method(_ncmesh.RefinementArray_Last)

    def DeleteLast(self):
        r"""DeleteLast(RefinementArray self)"""
        return _ncmesh.RefinementArray_DeleteLast(self)
    DeleteLast = _swig_new_instance_method(_ncmesh.RefinementArray_DeleteLast)

    def DeleteAll(self):
        r"""DeleteAll(RefinementArray self)"""
        return _ncmesh.RefinementArray_DeleteAll(self)
    DeleteAll = _swig_new_instance_method(_ncmesh.RefinementArray_DeleteAll)

    def Copy(self, copy):
        r"""Copy(RefinementArray self, RefinementArray copy)"""
        return _ncmesh.RefinementArray_Copy(self, copy)
    Copy = _swig_new_instance_method(_ncmesh.RefinementArray_Copy)

    def MakeRef(self, *args):
        r"""
        MakeRef(RefinementArray self, Refinement arg2, int arg3)
        MakeRef(RefinementArray self, RefinementArray master)
        """
        return _ncmesh.RefinementArray_MakeRef(self, *args)
    MakeRef = _swig_new_instance_method(_ncmesh.RefinementArray_MakeRef)

    def GetSubArray(self, offset, sa_size, sa):
        r"""GetSubArray(RefinementArray self, int offset, int sa_size, RefinementArray sa)"""
        return _ncmesh.RefinementArray_GetSubArray(self, offset, sa_size, sa)
    GetSubArray = _swig_new_instance_method(_ncmesh.RefinementArray_GetSubArray)

    def begin(self, *args):
        r"""
        begin(RefinementArray self) -> Refinement
        begin(RefinementArray self) -> Refinement
        """
        return _ncmesh.RefinementArray_begin(self, *args)
    begin = _swig_new_instance_method(_ncmesh.RefinementArray_begin)

    def end(self, *args):
        r"""
        end(RefinementArray self) -> Refinement
        end(RefinementArray self) -> Refinement
        """
        return _ncmesh.RefinementArray_end(self, *args)
    end = _swig_new_instance_method(_ncmesh.RefinementArray_end)

    def MemoryUsage(self):
        r"""MemoryUsage(RefinementArray self) -> long"""
        return _ncmesh.RefinementArray_MemoryUsage(self)
    MemoryUsage = _swig_new_instance_method(_ncmesh.RefinementArray_MemoryUsage)

    def Read(self, on_dev=True):
        r"""Read(RefinementArray self, bool on_dev=True) -> Refinement"""
        return _ncmesh.RefinementArray_Read(self, on_dev)
    Read = _swig_new_instance_method(_ncmesh.RefinementArray_Read)

    def HostRead(self):
        r"""HostRead(RefinementArray self) -> Refinement"""
        return _ncmesh.RefinementArray_HostRead(self)
    HostRead = _swig_new_instance_method(_ncmesh.RefinementArray_HostRead)

    def Write(self, on_dev=True):
        r"""Write(RefinementArray self, bool on_dev=True) -> Refinement"""
        return _ncmesh.RefinementArray_Write(self, on_dev)
    Write = _swig_new_instance_method(_ncmesh.RefinementArray_Write)

    def HostWrite(self):
        r"""HostWrite(RefinementArray self) -> Refinement"""
        return _ncmesh.RefinementArray_HostWrite(self)
    HostWrite = _swig_new_instance_method(_ncmesh.RefinementArray_HostWrite)

    def ReadWrite(self, on_dev=True):
        r"""ReadWrite(RefinementArray self, bool on_dev=True) -> Refinement"""
        return _ncmesh.RefinementArray_ReadWrite(self, on_dev)
    ReadWrite = _swig_new_instance_method(_ncmesh.RefinementArray_ReadWrite)

    def HostReadWrite(self):
        r"""HostReadWrite(RefinementArray self) -> Refinement"""
        return _ncmesh.RefinementArray_HostReadWrite(self)
    HostReadWrite = _swig_new_instance_method(_ncmesh.RefinementArray_HostReadWrite)

    def __setitem__(self, i, v):
        r"""__setitem__(RefinementArray self, int i, Refinement v)"""
        return _ncmesh.RefinementArray___setitem__(self, i, v)
    __setitem__ = _swig_new_instance_method(_ncmesh.RefinementArray___setitem__)

    def __getitem__(self, i):
        r"""__getitem__(RefinementArray self, int const i) -> Refinement"""
        return _ncmesh.RefinementArray___getitem__(self, i)
    __getitem__ = _swig_new_instance_method(_ncmesh.RefinementArray___getitem__)

    def Assign(self, *args):
        r"""
        Assign(RefinementArray self, Refinement arg2)
        Assign(RefinementArray self, Refinement a)
        """
        return _ncmesh.RefinementArray_Assign(self, *args)
    Assign = _swig_new_instance_method(_ncmesh.RefinementArray_Assign)

    def ToList(self):
        return [self[i] for i in range(self.Size())]



# Register RefinementArray in _ncmesh:
_ncmesh.RefinementArray_swigregister(RefinementArray)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _array
else:
    import _array

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _array.SWIG_PyInstanceMethod_New
_swig_new_static_method = _array.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import mfem._ser.mem_manager
class intArray(object):
    r"""Proxy of C++ mfem::Array< int > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(intArray self) -> intArray
        __init__(intArray self, mfem::MemoryType mt) -> intArray
        __init__(intArray self, int asize) -> intArray
        __init__(intArray self, int * _data) -> intArray
        __init__(intArray self, intArray src) -> intArray
        """
        _array.intArray_swiginit(self, _array.new_intArray(*args))

        if len(args) == 1 and isinstance(args[0], list):
            self.MakeDataOwner()



    __swig_destroy__ = _array.delete_intArray

    def GetData(self, *args):
        r"""
        GetData(intArray self) -> int
        GetData(intArray self) -> int const *
        """
        return _array.intArray_GetData(self, *args)
    GetData = _swig_new_instance_method(_array.intArray_GetData)

    def GetMemory(self, *args):
        r"""
        GetMemory(intArray self) -> mfem::Memory< int >
        GetMemory(intArray self) -> mfem::Memory< int > const &
        """
        return _array.intArray_GetMemory(self, *args)
    GetMemory = _swig_new_instance_method(_array.intArray_GetMemory)

    def UseDevice(self):
        r"""UseDevice(intArray self) -> bool"""
        return _array.intArray_UseDevice(self)
    UseDevice = _swig_new_instance_method(_array.intArray_UseDevice)

    def OwnsData(self):
        r"""OwnsData(intArray self) -> bool"""
        return _array.intArray_OwnsData(self)
    OwnsData = _swig_new_instance_method(_array.intArray_OwnsData)

    def StealData(self, p):
        r"""StealData(intArray self, int ** p)"""
        return _array.intArray_StealData(self, p)
    StealData = _swig_new_instance_method(_array.intArray_StealData)

    def LoseData(self):
        r"""LoseData(intArray self)"""
        return _array.intArray_LoseData(self)
    LoseData = _swig_new_instance_method(_array.intArray_LoseData)

    def MakeDataOwner(self):
        r"""MakeDataOwner(intArray self)"""
        return _array.intArray_MakeDataOwner(self)
    MakeDataOwner = _swig_new_instance_method(_array.intArray_MakeDataOwner)

    def Size(self):
        r"""Size(intArray self) -> int"""
        return _array.intArray_Size(self)
    Size = _swig_new_instance_method(_array.intArray_Size)

    def SetSize(self, *args):
        r"""
        SetSize(intArray self, int nsize)
        SetSize(intArray self, int nsize, int const & initval)
        SetSize(intArray self, int nsize, mfem::MemoryType mt)
        """
        return _array.intArray_SetSize(self, *args)
    SetSize = _swig_new_instance_method(_array.intArray_SetSize)

    def Capacity(self):
        r"""Capacity(intArray self) -> int"""
        return _array.intArray_Capacity(self)
    Capacity = _swig_new_instance_method(_array.intArray_Capacity)

    def Reserve(self, capacity):
        r"""Reserve(intArray self, int capacity)"""
        return _array.intArray_Reserve(self, capacity)
    Reserve = _swig_new_instance_method(_array.intArray_Reserve)

    def Append(self, *args):
        r"""
        Append(intArray self, int const & el) -> int
        Append(intArray self, int const * els, int nels) -> int
        Append(intArray self, intArray els) -> int
        """
        return _array.intArray_Append(self, *args)
    Append = _swig_new_instance_method(_array.intArray_Append)

    def Prepend(self, el):
        r"""Prepend(intArray self, int const & el) -> int"""
        return _array.intArray_Prepend(self, el)
    Prepend = _swig_new_instance_method(_array.intArray_Prepend)

    def Last(self, *args):
        r"""
        Last(intArray self) -> int
        Last(intArray self) -> int const &
        """
        return _array.intArray_Last(self, *args)
    Last = _swig_new_instance_method(_array.intArray_Last)

    def Union(self, el):
        r"""Union(intArray self, int const & el) -> int"""
        return _array.intArray_Union(self, el)
    Union = _swig_new_instance_method(_array.intArray_Union)

    def Find(self, el):
        r"""Find(intArray self, int const & el) -> int"""
        return _array.intArray_Find(self, el)
    Find = _swig_new_instance_method(_array.intArray_Find)

    def FindSorted(self, el):
        r"""FindSorted(intArray self, int const & el) -> int"""
        return _array.intArray_FindSorted(self, el)
    FindSorted = _swig_new_instance_method(_array.intArray_FindSorted)

    def DeleteLast(self):
        r"""DeleteLast(intArray self)"""
        return _array.intArray_DeleteLast(self)
    DeleteLast = _swig_new_instance_method(_array.intArray_DeleteLast)

    def DeleteFirst(self, el):
        r"""DeleteFirst(intArray self, int const & el)"""
        return _array.intArray_DeleteFirst(self, el)
    DeleteFirst = _swig_new_instance_method(_array.intArray_DeleteFirst)

    def DeleteAll(self):
        r"""DeleteAll(intArray self)"""
        return _array.intArray_DeleteAll(self)
    DeleteAll = _swig_new_instance_method(_array.intArray_DeleteAll)

    def Copy(self, copy):
        r"""Copy(intArray self, intArray copy)"""
        return _array.intArray_Copy(self, copy)
    Copy = _swig_new_instance_method(_array.intArray_Copy)

    def MakeRef(self, *args):
        r"""
        MakeRef(intArray self, int * arg2, int arg3)
        MakeRef(intArray self, intArray master)
        """
        return _array.intArray_MakeRef(self, *args)
    MakeRef = _swig_new_instance_method(_array.intArray_MakeRef)

    def GetSubArray(self, offset, sa_size, sa):
        r"""GetSubArray(intArray self, int offset, int sa_size, intArray sa)"""
        return _array.intArray_GetSubArray(self, offset, sa_size, sa)
    GetSubArray = _swig_new_instance_method(_array.intArray_GetSubArray)

    def Load(self, *args):
        r"""
        Load(intArray self, std::istream & _in, int fmt=0)
        Load(intArray self, int new_size, std::istream & _in)
        """
        return _array.intArray_Load(self, *args)
    Load = _swig_new_instance_method(_array.intArray_Load)

    def Max(self):
        r"""Max(intArray self) -> int"""
        return _array.intArray_Max(self)
    Max = _swig_new_instance_method(_array.intArray_Max)

    def Min(self):
        r"""Min(intArray self) -> int"""
        return _array.intArray_Min(self)
    Min = _swig_new_instance_method(_array.intArray_Min)

    def Sort(self):
        r"""Sort(intArray self)"""
        return _array.intArray_Sort(self)
    Sort = _swig_new_instance_method(_array.intArray_Sort)

    def Unique(self):
        r"""Unique(intArray self)"""
        return _array.intArray_Unique(self)
    Unique = _swig_new_instance_method(_array.intArray_Unique)

    def IsSorted(self):
        r"""IsSorted(intArray self) -> int"""
        return _array.intArray_IsSorted(self)
    IsSorted = _swig_new_instance_method(_array.intArray_IsSorted)

    def PartialSum(self):
        r"""PartialSum(intArray self)"""
        return _array.intArray_PartialSum(self)
    PartialSum = _swig_new_instance_method(_array.intArray_PartialSum)

    def Sum(self):
        r"""Sum(intArray self) -> int"""
        return _array.intArray_Sum(self)
    Sum = _swig_new_instance_method(_array.intArray_Sum)

    def begin(self, *args):
        r"""
        begin(intArray self) -> int
        begin(intArray self) -> int const *
        """
        return _array.intArray_begin(self, *args)
    begin = _swig_new_instance_method(_array.intArray_begin)

    def end(self, *args):
        r"""
        end(intArray self) -> int
        end(intArray self) -> int const *
        """
        return _array.intArray_end(self, *args)
    end = _swig_new_instance_method(_array.intArray_end)

    def MemoryUsage(self):
        r"""MemoryUsage(intArray self) -> long"""
        return _array.intArray_MemoryUsage(self)
    MemoryUsage = _swig_new_instance_method(_array.intArray_MemoryUsage)

    def Read(self, on_dev=True):
        r"""Read(intArray self, bool on_dev=True) -> int const *"""
        return _array.intArray_Read(self, on_dev)
    Read = _swig_new_instance_method(_array.intArray_Read)

    def HostRead(self):
        r"""HostRead(intArray self) -> int const *"""
        return _array.intArray_HostRead(self)
    HostRead = _swig_new_instance_method(_array.intArray_HostRead)

    def Write(self, on_dev=True):
        r"""Write(intArray self, bool on_dev=True) -> int *"""
        return _array.intArray_Write(self, on_dev)
    Write = _swig_new_instance_method(_array.intArray_Write)

    def HostWrite(self):
        r"""HostWrite(intArray self) -> int *"""
        return _array.intArray_HostWrite(self)
    HostWrite = _swig_new_instance_method(_array.intArray_HostWrite)

    def ReadWrite(self, on_dev=True):
        r"""ReadWrite(intArray self, bool on_dev=True) -> int *"""
        return _array.intArray_ReadWrite(self, on_dev)
    ReadWrite = _swig_new_instance_method(_array.intArray_ReadWrite)

    def HostReadWrite(self):
        r"""HostReadWrite(intArray self) -> int *"""
        return _array.intArray_HostReadWrite(self)
    HostReadWrite = _swig_new_instance_method(_array.intArray_HostReadWrite)

    def __setitem__(self, i, v):
        r"""__setitem__(intArray self, int i, int const v)"""
        return _array.intArray___setitem__(self, i, v)
    __setitem__ = _swig_new_instance_method(_array.intArray___setitem__)

    def __getitem__(self, i):
        r"""__getitem__(intArray self, int const i) -> int const &"""
        return _array.intArray___getitem__(self, i)
    __getitem__ = _swig_new_instance_method(_array.intArray___getitem__)

    def Assign(self, *args):
        r"""
        Assign(intArray self, int const * arg2)
        Assign(intArray self, int const & a)
        """
        return _array.intArray_Assign(self, *args)
    Assign = _swig_new_instance_method(_array.intArray_Assign)

    def ToList(self):
        return [self[i] for i in range(self.Size())]



    def Print(self, *args):
        r"""
        Print(intArray self, std::ostream & out=mfem::out, int width=4)
        Print(intArray self, char const * file, int precision=8)
        """
        return _array.intArray_Print(self, *args)
    Print = _swig_new_instance_method(_array.intArray_Print)

    def Save(self, *args):
        r"""
        Save(intArray self, std::ostream & out, int fmt=0)
        Save(intArray self, char const * file, int precision=8)
        Save(intArray self)
        """
        return _array.intArray_Save(self, *args)
    Save = _swig_new_instance_method(_array.intArray_Save)

# Register intArray in _array:
_array.intArray_swigregister(intArray)

class doubleArray(object):
    r"""Proxy of C++ mfem::Array< double > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(doubleArray self) -> doubleArray
        __init__(doubleArray self, mfem::MemoryType mt) -> doubleArray
        __init__(doubleArray self, int asize) -> doubleArray
        __init__(doubleArray self, double * _data) -> doubleArray
        __init__(doubleArray self, doubleArray src) -> doubleArray
        """
        _array.doubleArray_swiginit(self, _array.new_doubleArray(*args))

        if len(args) == 1 and isinstance(args[0], list):
            self.MakeDataOwner()



    __swig_destroy__ = _array.delete_doubleArray

    def GetData(self, *args):
        r"""
        GetData(doubleArray self) -> double
        GetData(doubleArray self) -> double const *
        """
        return _array.doubleArray_GetData(self, *args)
    GetData = _swig_new_instance_method(_array.doubleArray_GetData)

    def GetMemory(self, *args):
        r"""
        GetMemory(doubleArray self) -> mfem::Memory< double >
        GetMemory(doubleArray self) -> mfem::Memory< double > const &
        """
        return _array.doubleArray_GetMemory(self, *args)
    GetMemory = _swig_new_instance_method(_array.doubleArray_GetMemory)

    def UseDevice(self):
        r"""UseDevice(doubleArray self) -> bool"""
        return _array.doubleArray_UseDevice(self)
    UseDevice = _swig_new_instance_method(_array.doubleArray_UseDevice)

    def OwnsData(self):
        r"""OwnsData(doubleArray self) -> bool"""
        return _array.doubleArray_OwnsData(self)
    OwnsData = _swig_new_instance_method(_array.doubleArray_OwnsData)

    def StealData(self, p):
        r"""StealData(doubleArray self, double ** p)"""
        return _array.doubleArray_StealData(self, p)
    StealData = _swig_new_instance_method(_array.doubleArray_StealData)

    def LoseData(self):
        r"""LoseData(doubleArray self)"""
        return _array.doubleArray_LoseData(self)
    LoseData = _swig_new_instance_method(_array.doubleArray_LoseData)

    def MakeDataOwner(self):
        r"""MakeDataOwner(doubleArray self)"""
        return _array.doubleArray_MakeDataOwner(self)
    MakeDataOwner = _swig_new_instance_method(_array.doubleArray_MakeDataOwner)

    def Size(self):
        r"""Size(doubleArray self) -> int"""
        return _array.doubleArray_Size(self)
    Size = _swig_new_instance_method(_array.doubleArray_Size)

    def SetSize(self, *args):
        r"""
        SetSize(doubleArray self, int nsize)
        SetSize(doubleArray self, int nsize, double const & initval)
        SetSize(doubleArray self, int nsize, mfem::MemoryType mt)
        """
        return _array.doubleArray_SetSize(self, *args)
    SetSize = _swig_new_instance_method(_array.doubleArray_SetSize)

    def Capacity(self):
        r"""Capacity(doubleArray self) -> int"""
        return _array.doubleArray_Capacity(self)
    Capacity = _swig_new_instance_method(_array.doubleArray_Capacity)

    def Reserve(self, capacity):
        r"""Reserve(doubleArray self, int capacity)"""
        return _array.doubleArray_Reserve(self, capacity)
    Reserve = _swig_new_instance_method(_array.doubleArray_Reserve)

    def Append(self, *args):
        r"""
        Append(doubleArray self, double const & el) -> int
        Append(doubleArray self, double const * els, int nels) -> int
        Append(doubleArray self, doubleArray els) -> int
        """
        return _array.doubleArray_Append(self, *args)
    Append = _swig_new_instance_method(_array.doubleArray_Append)

    def Prepend(self, el):
        r"""Prepend(doubleArray self, double const & el) -> int"""
        return _array.doubleArray_Prepend(self, el)
    Prepend = _swig_new_instance_method(_array.doubleArray_Prepend)

    def Last(self, *args):
        r"""
        Last(doubleArray self) -> double
        Last(doubleArray self) -> double const &
        """
        return _array.doubleArray_Last(self, *args)
    Last = _swig_new_instance_method(_array.doubleArray_Last)

    def Union(self, el):
        r"""Union(doubleArray self, double const & el) -> int"""
        return _array.doubleArray_Union(self, el)
    Union = _swig_new_instance_method(_array.doubleArray_Union)

    def Find(self, el):
        r"""Find(doubleArray self, double const & el) -> int"""
        return _array.doubleArray_Find(self, el)
    Find = _swig_new_instance_method(_array.doubleArray_Find)

    def FindSorted(self, el):
        r"""FindSorted(doubleArray self, double const & el) -> int"""
        return _array.doubleArray_FindSorted(self, el)
    FindSorted = _swig_new_instance_method(_array.doubleArray_FindSorted)

    def DeleteLast(self):
        r"""DeleteLast(doubleArray self)"""
        return _array.doubleArray_DeleteLast(self)
    DeleteLast = _swig_new_instance_method(_array.doubleArray_DeleteLast)

    def DeleteFirst(self, el):
        r"""DeleteFirst(doubleArray self, double const & el)"""
        return _array.doubleArray_DeleteFirst(self, el)
    DeleteFirst = _swig_new_instance_method(_array.doubleArray_DeleteFirst)

    def DeleteAll(self):
        r"""DeleteAll(doubleArray self)"""
        return _array.doubleArray_DeleteAll(self)
    DeleteAll = _swig_new_instance_method(_array.doubleArray_DeleteAll)

    def Copy(self, copy):
        r"""Copy(doubleArray self, doubleArray copy)"""
        return _array.doubleArray_Copy(self, copy)
    Copy = _swig_new_instance_method(_array.doubleArray_Copy)

    def MakeRef(self, *args):
        r"""
        MakeRef(doubleArray self, double * arg2, int arg3)
        MakeRef(doubleArray self, doubleArray master)
        """
        return _array.doubleArray_MakeRef(self, *args)
    MakeRef = _swig_new_instance_method(_array.doubleArray_MakeRef)

    def GetSubArray(self, offset, sa_size, sa):
        r"""GetSubArray(doubleArray self, int offset, int sa_size, doubleArray sa)"""
        return _array.doubleArray_GetSubArray(self, offset, sa_size, sa)
    GetSubArray = _swig_new_instance_method(_array.doubleArray_GetSubArray)

    def Load(self, *args):
        r"""
        Load(doubleArray self, std::istream & _in, int fmt=0)
        Load(doubleArray self, int new_size, std::istream & _in)
        """
        return _array.doubleArray_Load(self, *args)
    Load = _swig_new_instance_method(_array.doubleArray_Load)

    def Max(self):
        r"""Max(doubleArray self) -> double"""
        return _array.doubleArray_Max(self)
    Max = _swig_new_instance_method(_array.doubleArray_Max)

    def Min(self):
        r"""Min(doubleArray self) -> double"""
        return _array.doubleArray_Min(self)
    Min = _swig_new_instance_method(_array.doubleArray_Min)

    def Sort(self):
        r"""Sort(doubleArray self)"""
        return _array.doubleArray_Sort(self)
    Sort = _swig_new_instance_method(_array.doubleArray_Sort)

    def Unique(self):
        r"""Unique(doubleArray self)"""
        return _array.doubleArray_Unique(self)
    Unique = _swig_new_instance_method(_array.doubleArray_Unique)

    def IsSorted(self):
        r"""IsSorted(doubleArray self) -> int"""
        return _array.doubleArray_IsSorted(self)
    IsSorted = _swig_new_instance_method(_array.doubleArray_IsSorted)

    def PartialSum(self):
        r"""PartialSum(doubleArray self)"""
        return _array.doubleArray_PartialSum(self)
    PartialSum = _swig_new_instance_method(_array.doubleArray_PartialSum)

    def Sum(self):
        r"""Sum(doubleArray self) -> double"""
        return _array.doubleArray_Sum(self)
    Sum = _swig_new_instance_method(_array.doubleArray_Sum)

    def begin(self, *args):
        r"""
        begin(doubleArray self) -> double
        begin(doubleArray self) -> double const *
        """
        return _array.doubleArray_begin(self, *args)
    begin = _swig_new_instance_method(_array.doubleArray_begin)

    def end(self, *args):
        r"""
        end(doubleArray self) -> double
        end(doubleArray self) -> double const *
        """
        return _array.doubleArray_end(self, *args)
    end = _swig_new_instance_method(_array.doubleArray_end)

    def MemoryUsage(self):
        r"""MemoryUsage(doubleArray self) -> long"""
        return _array.doubleArray_MemoryUsage(self)
    MemoryUsage = _swig_new_instance_method(_array.doubleArray_MemoryUsage)

    def Read(self, on_dev=True):
        r"""Read(doubleArray self, bool on_dev=True) -> double const *"""
        return _array.doubleArray_Read(self, on_dev)
    Read = _swig_new_instance_method(_array.doubleArray_Read)

    def HostRead(self):
        r"""HostRead(doubleArray self) -> double const *"""
        return _array.doubleArray_HostRead(self)
    HostRead = _swig_new_instance_method(_array.doubleArray_HostRead)

    def Write(self, on_dev=True):
        r"""Write(doubleArray self, bool on_dev=True) -> double *"""
        return _array.doubleArray_Write(self, on_dev)
    Write = _swig_new_instance_method(_array.doubleArray_Write)

    def HostWrite(self):
        r"""HostWrite(doubleArray self) -> double *"""
        return _array.doubleArray_HostWrite(self)
    HostWrite = _swig_new_instance_method(_array.doubleArray_HostWrite)

    def ReadWrite(self, on_dev=True):
        r"""ReadWrite(doubleArray self, bool on_dev=True) -> double *"""
        return _array.doubleArray_ReadWrite(self, on_dev)
    ReadWrite = _swig_new_instance_method(_array.doubleArray_ReadWrite)

    def HostReadWrite(self):
        r"""HostReadWrite(doubleArray self) -> double *"""
        return _array.doubleArray_HostReadWrite(self)
    HostReadWrite = _swig_new_instance_method(_array.doubleArray_HostReadWrite)

    def __setitem__(self, i, v):
        r"""__setitem__(doubleArray self, int i, double const v)"""
        return _array.doubleArray___setitem__(self, i, v)
    __setitem__ = _swig_new_instance_method(_array.doubleArray___setitem__)

    def __getitem__(self, i):
        r"""__getitem__(doubleArray self, int const i) -> double const &"""
        return _array.doubleArray___getitem__(self, i)
    __getitem__ = _swig_new_instance_method(_array.doubleArray___getitem__)

    def Assign(self, *args):
        r"""
        Assign(doubleArray self, double const * arg2)
        Assign(doubleArray self, double const & a)
        """
        return _array.doubleArray_Assign(self, *args)
    Assign = _swig_new_instance_method(_array.doubleArray_Assign)

    def ToList(self):
        return [self[i] for i in range(self.Size())]



    def Print(self, *args):
        r"""
        Print(doubleArray self, std::ostream & out=mfem::out, int width=4)
        Print(doubleArray self, char const * file, int precision=8)
        """
        return _array.doubleArray_Print(self, *args)
    Print = _swig_new_instance_method(_array.doubleArray_Print)

    def Save(self, *args):
        r"""
        Save(doubleArray self, std::ostream & out, int fmt=0)
        Save(doubleArray self, char const * file, int precision=8)
        Save(doubleArray self)
        """
        return _array.doubleArray_Save(self, *args)
    Save = _swig_new_instance_method(_array.doubleArray_Save)

# Register doubleArray in _array:
_array.doubleArray_swigregister(doubleArray)


def doubleSwap(*args):
    r"""
    doubleSwap(doubleArray arg1, doubleArray arg2)
    doubleSwap(mfem::Array2D< double > & arg1, mfem::Array2D< double > & arg2)
    doubleSwap(double & a, double & b)
    """
    return _array.doubleSwap(*args)
doubleSwap = _array.doubleSwap

def intSwap(*args):
    r"""
    intSwap(intArray arg1, intArray arg2)
    intSwap(mfem::Array2D< int > & arg1, mfem::Array2D< int > & arg2)
    intSwap(int & a, int & b)
    """
    return _array.intSwap(*args)
intSwap = _array.intSwap


